from typing import TYPE_CHECKING, Any

import pydantic

from vkbottle_types.base_model import BaseModel

from .enums import GroupEventType
from .objects import group_event_objects

if TYPE_CHECKING:
    from vkbottle import ABCAPI, API  # type: ignore


class BaseGroupEvent(BaseModel):
    object: group_event_objects.BaseEventObject | None
    type: GroupEventType | None = None
    secret: str | None = None
    event_id: str | None = None
    group_id: int | None = None
    unprepared_ctx_api: Any | None = None

    model_config = pydantic.ConfigDict(frozen=False)

    @property
    def ctx_api(self) -> "ABCAPI | API":
        if self.unprepared_ctx_api is None:
            raise AssertionError
        return self.unprepared_ctx_api


class MessageNew(BaseGroupEvent):
    object: group_event_objects.MessageNewObject


class MessageReply(BaseGroupEvent):
    object: group_event_objects.MessageReplyObject


class MessageEdit(BaseGroupEvent):
    object: group_event_objects.MessageEditObject


class MessageAllow(BaseGroupEvent):
    object: group_event_objects.MessageAllowObject


class MessageDeny(BaseGroupEvent):
    object: group_event_objects.MessageDenyObject


class MessageTypingState(BaseGroupEvent):
    object: group_event_objects.MessageTypingStateObject


class MessageEvent(BaseGroupEvent):
    object: group_event_objects.MessageEventObject


class PhotoNew(BaseGroupEvent):
    object: group_event_objects.PhotoNewObject


class PhotoCommentNew(BaseGroupEvent):
    object: group_event_objects.PhotoCommentNewObject


class PhotoCommentEdit(BaseGroupEvent):
    object: group_event_objects.PhotoCommentNewObject


class PhotoCommentRestore(BaseGroupEvent):
    object: group_event_objects.PhotoCommentRestoreObject


class PhotoCommentDelete(BaseGroupEvent):
    object: group_event_objects.PhotoCommentDeleteObject


class AudioNew(BaseGroupEvent):
    object: group_event_objects.AudioNewObject


class VideoNew(BaseGroupEvent):
    object: group_event_objects.VideoNewObject


class VideoCommentNew(BaseGroupEvent):
    object: group_event_objects.VideoCommentNewObject


class VideoCommentEdit(BaseGroupEvent):
    object: group_event_objects.VideoCommentEditObject


class VideoCommentRestore(BaseGroupEvent):
    object: group_event_objects.VideoCommentRestoreObject


class LeadFormsNew(BaseGroupEvent):
    object: group_event_objects.LeadFormsNewObject


class VideoCommentDelete(BaseGroupEvent):
    object: group_event_objects.VideoCommentDeleteObject


class WallPostNew(BaseGroupEvent):
    object: group_event_objects.WallPostNewObject


class WallRepost(BaseGroupEvent):
    object: group_event_objects.WallRepostObject


class WallReplyNew(BaseGroupEvent):
    object: group_event_objects.WallReplyNewObject


class WallReplyEdit(BaseGroupEvent):
    object: group_event_objects.WallReplyRestoreObject


class WallReplyRestore(BaseGroupEvent):
    object: group_event_objects.WallReplyRestoreObject


class WallReplyDelete(BaseGroupEvent):
    object: group_event_objects.WallReplyDeleteObject


class LikeAdd(BaseGroupEvent):
    object: group_event_objects.LikeAddObject


class LikeRemove(BaseGroupEvent):
    object: group_event_objects.LikeRemoveObject


class BoardPostNew(BaseGroupEvent):
    object: group_event_objects.BoardPostNewObject


class BoardPostEdit(BaseGroupEvent):
    object: group_event_objects.BoardPostEditObject


class BoardPostRestore(BaseGroupEvent):
    object: group_event_objects.BoardPostRestoreObject


class BoardPostDelete(BaseGroupEvent):
    object: group_event_objects.BoardPostDeleteObject


class MarketCommentNew(BaseGroupEvent):
    object: group_event_objects.MarketCommentNewObject


class MarketCommentEdit(BaseGroupEvent):
    object: group_event_objects.MarketCommentEditObject


class MarketCommentRestore(BaseGroupEvent):
    object: group_event_objects.MarketCommentRestoreObject


class MarketCommentDelete(BaseGroupEvent):
    object: group_event_objects.MarketCommentDeleteObject


class MarketOrderNew(BaseGroupEvent):
    object: group_event_objects.MarketOrderNewObject


class MarketOrderEdit(BaseGroupEvent):
    object: group_event_objects.MarketOrderEditObject


class GroupLeave(BaseGroupEvent):
    object: group_event_objects.GroupLeaveObject


class GroupJoin(BaseGroupEvent):
    object: group_event_objects.GroupJoinObject


class UserBlock(BaseGroupEvent):
    object: group_event_objects.UserBlockObject


class UserUnblock(BaseGroupEvent):
    object: group_event_objects.UserUnblockObject


class PollVoteNew(BaseGroupEvent):
    object: group_event_objects.PollVoteNewObject


class GroupOfficersEdit(BaseGroupEvent):
    object: group_event_objects.GroupOfficersEditObject


class GroupChangeSettings(BaseGroupEvent):
    object: group_event_objects.GroupChangeSettingsObject


class GroupChangePhoto(BaseGroupEvent):
    object: group_event_objects.GroupChangePhotoObject


class VkpayTransaction(BaseGroupEvent):
    object: group_event_objects.VkpayTransactionObject


class AppPayload(BaseGroupEvent):
    object: group_event_objects.AppPayloadObject


class DonutSubscriptionCreate(BaseGroupEvent):
    object: group_event_objects.DonutSubscriptionCreateObject


class DonutSubscriptionProlonged(BaseGroupEvent):
    object: group_event_objects.DonutSubscriptionProlongedObject


class DonutSubscriptionExpired(BaseGroupEvent):
    object: group_event_objects.DonutSubscriptionExpiredObject


class DonutSubscriptionCancelled(BaseGroupEvent):
    object: group_event_objects.DonutSubscriptionCancelledObject


class DonutSubscriptionPriceChanged(BaseGroupEvent):
    object: group_event_objects.DonutSubscriptionPriceChangedObject


class DonutMoneyWithdraw(BaseGroupEvent):
    object: group_event_objects.DonutMoneyWithdrawObject


class DonutMoneyWithdrawError(BaseGroupEvent):
    object: group_event_objects.DonutMoneyWithdrawErrorObject


class MessageReactionEvent(BaseGroupEvent):
    object: group_event_objects.MessageReactionEventObject


class MessageRead(BaseGroupEvent):
    object: group_event_objects.MessageReadObject


__all__ = (
    "AppPayload",
    "AudioNew",
    "BaseGroupEvent",
    "BoardPostDelete",
    "BoardPostEdit",
    "BoardPostNew",
    "BoardPostRestore",
    "DonutMoneyWithdraw",
    "DonutMoneyWithdrawError",
    "DonutSubscriptionCancelled",
    "DonutSubscriptionCreate",
    "DonutSubscriptionExpired",
    "DonutSubscriptionPriceChanged",
    "DonutSubscriptionProlonged",
    "GroupChangePhoto",
    "GroupChangeSettings",
    "GroupJoin",
    "GroupLeave",
    "GroupOfficersEdit",
    "LeadFormsNew",
    "LikeAdd",
    "LikeRemove",
    "MarketCommentDelete",
    "MarketCommentEdit",
    "MarketCommentNew",
    "MarketCommentRestore",
    "MarketOrderEdit",
    "MarketOrderNew",
    "MessageAllow",
    "MessageDeny",
    "MessageEdit",
    "MessageEvent",
    "MessageNew",
    "MessageReactionEvent",
    "MessageRead",
    "MessageReply",
    "MessageTypingState",
    "PhotoCommentDelete",
    "PhotoCommentEdit",
    "PhotoCommentNew",
    "PhotoCommentRestore",
    "PhotoNew",
    "PollVoteNew",
    "UserBlock",
    "UserUnblock",
    "VideoCommentDelete",
    "VideoCommentEdit",
    "VideoCommentNew",
    "VideoCommentRestore",
    "VideoNew",
    "VkpayTransaction",
    "WallPostNew",
    "WallReplyDelete",
    "WallReplyEdit",
    "WallReplyNew",
    "WallReplyRestore",
    "WallRepost",
)



from vkbottle_types.base_model import Field
from vkbottle_types.objects import StatsPeriod, StatsWallpostStat
from vkbottle_types.responses.base_response import BaseResponse


class StatsGetPostReachResponse(BaseResponse):
    response: list["StatsWallpostStat"] = Field()


class StatsGetResponse(BaseResponse):
    response: list["StatsPeriod"] = Field()



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import StreamingStats
from vkbottle_types.responses.base_response import BaseResponse


class StreamingGetServerUrlResponseModel(BaseModel):
    endpoint: str | None = Field(
        default=None,
    )
    key: str | None = Field(
        default=None,
    )


class StreamingGetServerUrlResponse(BaseResponse):
    response: "StreamingGetServerUrlResponseModel" = Field()


class StreamingGetStatsResponse(BaseResponse):
    response: list["StreamingStats"] = Field()


class StreamingGetStemResponseModel(BaseModel):
    stem: str | None = Field(
        default=None,
    )


class StreamingGetStemResponse(BaseResponse):
    response: "StreamingGetStemResponseModel" = Field()



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import (
    BaseImage,
    GroupsGroupFull,
    PhotosPhoto,
    PhotosPhotoAlbumFull,
    PhotosPhotoTag,
    PhotosPhotoUpload,
    PhotosPhotoXtrTagInfo,
    UsersUserFull,
    WallWallComment,
)
from vkbottle_types.responses.base_response import BaseResponse


class PhotosCopyResponse(BaseResponse):
    response: int = Field()


class PhotosCreateAlbumResponse(BaseResponse):
    response: "PhotosPhotoAlbumFull" = Field()


class PhotosCreateCommentResponse(BaseResponse):
    response: int = Field()


class PhotosGetAlbumsCountResponse(BaseResponse):
    response: int = Field()


class PhotosGetAlbumsResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhotoAlbumFull"] = Field()


class PhotosGetAlbumsResponse(BaseResponse):
    response: "PhotosGetAlbumsResponseModel" = Field()


class PhotosGetAllCommentsResponseModel(BaseModel):
    count: int = Field()
    items: list["WallWallComment"] = Field()


class PhotosGetAllCommentsResponse(BaseResponse):
    response: "PhotosGetAllCommentsResponseModel" = Field()


class PhotosGetAllResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhoto"] = Field()
    more: bool | None = Field(
        default=None,
    )


class PhotosGetAllResponse(BaseResponse):
    response: "PhotosGetAllResponseModel" = Field()


class PhotosGetByIdResponse(BaseResponse):
    response: list["PhotosPhoto"] = Field()


class PhotosGetCommentsExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list["WallWallComment"] = Field()
    profiles: list["UsersUserFull"] = Field()
    groups: list["GroupsGroupFull"] = Field()
    real_offset: int | None = Field(
        default=None,
    )


class PhotosGetCommentsExtendedResponse(BaseResponse):
    response: "PhotosGetCommentsExtendedResponseModel" = Field()


class PhotosGetCommentsResponseModel(BaseModel):
    count: int = Field()
    items: list["WallWallComment"] = Field()
    real_offset: int | None = Field(
        default=None,
    )


class PhotosGetCommentsResponse(BaseResponse):
    response: "PhotosGetCommentsResponseModel" = Field()


class PhotosGetMessagesUploadServerResponse(BaseResponse):
    response: "PhotosPhotoUpload" = Field()


class PhotosGetNewTagsResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhotoXtrTagInfo"] = Field()


class PhotosGetNewTagsResponse(BaseResponse):
    response: "PhotosGetNewTagsResponseModel" = Field()


class PhotosGetTagsResponse(BaseResponse):
    response: list["PhotosPhotoTag"] = Field()


class PhotosGetUploadServerResponse(BaseResponse):
    response: "PhotosPhotoUpload" = Field()


class PhotosGetUserPhotosResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhoto"] = Field()
    next_from: str | None = Field(
        default=None,
    )


class PhotosGetUserPhotosResponse(BaseResponse):
    response: "PhotosGetUserPhotosResponseModel" = Field()


class PhotosGetWallUploadServerResponse(BaseResponse):
    response: "PhotosPhotoUpload" = Field()


class PhotosGetResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhoto"] = Field()
    next_from: str | None = Field(
        default=None,
    )


class PhotosGetResponse(BaseResponse):
    response: "PhotosGetResponseModel" = Field()


class PhotosMarketAlbumUploadResponseModel(BaseModel):
    gid: int | None = Field(
        default=None,
    )
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosMarketAlbumUploadResponse(BaseResponse):
    response: "PhotosMarketAlbumUploadResponseModel" = Field()


class PhotosMarketUploadResponseModel(BaseModel):
    crop_data: str | None = Field(
        default=None,
    )
    crop_hash: str | None = Field(
        default=None,
    )
    group_id: int | None = Field(
        default=None,
    )
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosMarketUploadResponse(BaseResponse):
    response: "PhotosMarketUploadResponseModel" = Field()


class PhotosMessageUploadResponseModel(BaseModel):
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosMessageUploadResponse(BaseResponse):
    response: "PhotosMessageUploadResponseModel" = Field()


class PhotosOwnerCoverUploadResponseModel(BaseModel):
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )


class PhotosOwnerCoverUploadResponse(BaseResponse):
    response: "PhotosOwnerCoverUploadResponseModel" = Field()


class PhotosOwnerUploadResponseModel(BaseModel):
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosOwnerUploadResponse(BaseResponse):
    response: "PhotosOwnerUploadResponseModel" = Field()


class PhotosPhotoUploadResponseModel(BaseModel):
    aid: int | None = Field(
        default=None,
    )
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    photos_list: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosPhotoUploadResponse(BaseResponse):
    response: "PhotosPhotoUploadResponseModel" = Field()


class PhotosPutTagResponse(BaseResponse):
    response: int = Field()


class PhotosSaveMarketAlbumPhotoResponse(BaseResponse):
    response: list["PhotosPhoto"] = Field()


class PhotosSaveMessagesPhotoResponse(BaseResponse):
    response: list["PhotosPhoto"] = Field()


class PhotosSaveOwnerCoverPhotoResponseModel(BaseModel):
    images: list["BaseImage"] | None = Field(
        default=None,
    )


class PhotosSaveOwnerCoverPhotoResponse(BaseResponse):
    response: "PhotosSaveOwnerCoverPhotoResponseModel" = Field()


class PhotosSaveOwnerPhotoResponseModel(BaseModel):
    photo_hash: str = Field()
    photo_src: str = Field()
    photo_src_big: str | None = Field(
        default=None,
    )
    photo_src_small: str | None = Field(
        default=None,
    )
    saved: int | None = Field(
        default=None,
    )
    post_id: int | None = Field(
        default=None,
    )


class PhotosSaveOwnerPhotoResponse(BaseResponse):
    response: "PhotosSaveOwnerPhotoResponseModel" = Field()


class PhotosSaveWallPhotoResponse(BaseResponse):
    response: list["PhotosPhoto"] = Field()


class PhotosSaveResponse(BaseResponse):
    response: list["PhotosPhoto"] = Field()


class PhotosSearchResponseModel(BaseModel):
    count: int = Field()
    items: list["PhotosPhoto"] = Field()


class PhotosSearchResponse(BaseResponse):
    response: "PhotosSearchResponseModel" = Field()


class PhotosWallUploadResponseModel(BaseModel):
    hash: str | None = Field(
        default=None,
    )
    photo: str | None = Field(
        default=None,
    )
    server: int | None = Field(
        default=None,
    )


class PhotosWallUploadResponse(BaseResponse):
    response: "PhotosWallUploadResponseModel" = Field()



from vkbottle_types.base_model import Field
from vkbottle_types.objects import StorageValue
from vkbottle_types.responses.base_response import BaseResponse


class StorageGetKeysResponse(BaseResponse):
    response: list[str] = Field()


class StorageGetResponse(BaseResponse):
    response: list["StorageValue"] = Field()



from typing import TypeAlias

from .bot_events import (
    AppPayload,
    AudioNew,
    BoardPostDelete,
    BoardPostEdit,
    BoardPostNew,
    BoardPostRestore,
    DonutMoneyWithdraw,
    DonutMoneyWithdrawError,
    DonutSubscriptionCancelled,
    DonutSubscriptionCreate,
    DonutSubscriptionExpired,
    DonutSubscriptionPriceChanged,
    DonutSubscriptionProlonged,
    GroupChangePhoto,
    GroupChangeSettings,
    GroupJoin,
    GroupLeave,
    GroupOfficersEdit,
    LeadFormsNew,
    LikeAdd,
    LikeRemove,
    MarketCommentDelete,
    MarketCommentEdit,
    MarketCommentNew,
    MarketCommentRestore,
    MarketOrderEdit,
    MarketOrderNew,
    MessageAllow,
    MessageDeny,
    MessageEdit,
    MessageEvent,
    MessageNew,
    MessageReactionEvent,
    MessageRead,
    MessageReply,
    MessageTypingState,
    PhotoCommentDelete,
    PhotoCommentEdit,
    PhotoCommentNew,
    PhotoCommentRestore,
    PhotoNew,
    PollVoteNew,
    UserBlock,
    UserUnblock,
    VideoCommentDelete,
    VideoCommentEdit,
    VideoCommentNew,
    VideoCommentRestore,
    VideoNew,
    VkpayTransaction,
    WallPostNew,
    WallReplyDelete,
    WallReplyEdit,
    WallReplyNew,
    WallReplyRestore,
    WallRepost,
)


class GroupTypes:
    MessageNew: TypeAlias = MessageNew
    MessageReply: TypeAlias = MessageReply
    MessageEdit: TypeAlias = MessageEdit
    MessageRead: TypeAlias = MessageRead
    MessageAllow: TypeAlias = MessageAllow
    MessageDeny: TypeAlias = MessageDeny
    MessageReactionEvent: TypeAlias = MessageReactionEvent
    MessageTypingState: TypeAlias = MessageTypingState
    MessageEvent: TypeAlias = MessageEvent
    PhotoNew: TypeAlias = PhotoNew
    PhotoCommentNew: TypeAlias = PhotoCommentNew
    PhotoCommentEdit: TypeAlias = PhotoCommentEdit
    PhotoCommentRestore: TypeAlias = PhotoCommentRestore
    PhotoCommentDelete: TypeAlias = PhotoCommentDelete
    AudioNew: TypeAlias = AudioNew
    VideoNew: TypeAlias = VideoNew
    VideoCommentNew: TypeAlias = VideoCommentNew
    VideoCommentEdit: TypeAlias = VideoCommentEdit
    VideoCommentRestore: TypeAlias = VideoCommentRestore
    VideoCommentDelete: TypeAlias = VideoCommentDelete
    WallPostNew: TypeAlias = WallPostNew
    WallRepost: TypeAlias = WallRepost
    WallReplyNew: TypeAlias = WallReplyNew
    WallReplyEdit: TypeAlias = WallReplyEdit
    WallReplyRestore: TypeAlias = WallReplyRestore
    WallReplyDelete: TypeAlias = WallReplyDelete
    LeadFormsNew: TypeAlias = LeadFormsNew
    LikeAdd: TypeAlias = LikeAdd
    LikeRemove: TypeAlias = LikeRemove
    BoardPostNew: TypeAlias = BoardPostNew
    BoardPostEdit: TypeAlias = BoardPostEdit
    BoardPostRestore: TypeAlias = BoardPostRestore
    BoardPostDelete: TypeAlias = BoardPostDelete
    MarketCommentNew: TypeAlias = MarketCommentNew
    MarketCommentEdit: TypeAlias = MarketCommentEdit
    MarketCommentRestore: TypeAlias = MarketCommentRestore
    MarketCommentDelete: TypeAlias = MarketCommentDelete
    MarketOrderNew: TypeAlias = MarketOrderNew
    MarketOrderEdit: TypeAlias = MarketOrderEdit
    GroupLeave: TypeAlias = GroupLeave
    GroupJoin: TypeAlias = GroupJoin
    UserBlock: TypeAlias = UserBlock
    UserUnblock: TypeAlias = UserUnblock
    PollVoteNew: TypeAlias = PollVoteNew
    GroupOfficersEdit: TypeAlias = GroupOfficersEdit
    GroupChangeSettings: TypeAlias = GroupChangeSettings
    GroupChangePhoto: TypeAlias = GroupChangePhoto
    VkpayTransaction: TypeAlias = VkpayTransaction
    AppPayload: TypeAlias = AppPayload
    DonutSubscriptionCreate: TypeAlias = DonutSubscriptionCreate
    DonutSubscriptionProlonged: TypeAlias = DonutSubscriptionProlonged
    DonutSubscriptionExpired: TypeAlias = DonutSubscriptionExpired
    DonutSubscriptionCancelled: TypeAlias = DonutSubscriptionCancelled
    DonutSubscriptionPriceChanged: TypeAlias = DonutSubscriptionPriceChanged
    DonutMoneyWithdraw: TypeAlias = DonutMoneyWithdraw
    DonutMoneyWithdrawError: TypeAlias = DonutMoneyWithdrawError


__all__ = ("GroupTypes",)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import StreamingStats
from vkbottle_types.responses.streaming import *  # noqa: F401,F403  # type: ignore


class StreamingCategory(BaseCategory):
    async def get_server_url(
        self,
        **kwargs: typing.Any,
    ) -> StreamingGetServerUrlResponseModel:
        """Method `streaming.getServerUrl()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("streaming.getServerUrl", params)
        model = StreamingGetServerUrlResponse
        return model(**response).response

    async def get_stats(
        self,
        end_time: int | None = None,
        interval: str | None = None,
        start_time: int | None = None,
        type: str | None = None,
        **kwargs: typing.Any,
    ) -> list[StreamingStats]:
        """Method `streaming.getStats()`

        :param end_time:
        :param interval:
        :param start_time:
        :param type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("streaming.getStats", params)
        model = StreamingGetStatsResponse
        return model(**response).response

    async def get_stem(
        self,
        word: str,
        **kwargs: typing.Any,
    ) -> StreamingGetStemResponseModel:
        """Method `streaming.getStem()`

        :param word:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("streaming.getStem", params)
        model = StreamingGetStemResponse
        return model(**response).response


__all__ = ("StreamingCategory",)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.responses.auth import *  # noqa: F401,F403  # type: ignore


class AuthCategory(BaseCategory):
    async def restore(
        self,
        last_name: str,
        phone: str,
        **kwargs: typing.Any,
    ) -> AuthRestoreResponseModel:
        """Method `auth.restore()`

        :param last_name: User last name.
        :param phone: User phone number.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("auth.restore", params)
        model = AuthRestoreResponse
        return model(**response).response


__all__ = ("AuthCategory",)



import msgspec
from typing_extensions import TYPE_CHECKING, Any, Self

from .objects import user_event_objects

if TYPE_CHECKING:
    from vkbottle import ABCAPI, API  # type: ignore


class BaseUserEvent(msgspec.Struct, omit_defaults=True):
    object: Any | None
    unprepared_ctx_api: Any | None = None

    @classmethod
    def from_raw(cls, data: bytes) -> Self:
        return msgspec.json.decode(data, type=cls)

    @classmethod
    def parse(cls, obj: list[Any]) -> Self:
        return msgspec.convert({"object": obj}, type=cls)

    def to_dict(self) -> dict[str, Any]:
        return msgspec.structs.asdict(self)

    @property
    def ctx_api(self) -> "ABCAPI | API":
        if self.unprepared_ctx_api is None:
            raise AssertionError
        return self.unprepared_ctx_api


class RawUserEvent(BaseUserEvent):
    object: list[Any]


class MessageFlagsReplace(BaseUserEvent):
    object: user_event_objects.MessageFlagsReplaceObject


class MessageSetFlags(BaseUserEvent):
    object: user_event_objects.MessageSetFlagsObject


class MessageResetFlags(BaseUserEvent):
    object: user_event_objects.MessageResetFlagsObject


class MessageNew(BaseUserEvent):
    object: user_event_objects.MessageNewObject


class MessagesDelete(BaseUserEvent):
    object: user_event_objects.MessagesDeleteObject


class MessagesRestore(BaseUserEvent):
    object: user_event_objects.MessagesRestoreObject


class MessageEdit(BaseUserEvent):
    object: user_event_objects.MessageEditObject


class ChatVoiceMessageStates(BaseUserEvent):
    object: user_event_objects.ChatVoiceMessageStatesObject


class ChatEdit(BaseUserEvent):
    object: user_event_objects.ChatEditObject


class ChatInfoEdit(BaseUserEvent):
    object: user_event_objects.ChatInfoEditObject


class ChatTypingState(BaseUserEvent):
    object: user_event_objects.ConversationTypingStateObject


class DialogTypingState(BaseUserEvent):
    object: user_event_objects.DialogTypingStateObject


class UsersTypingState(BaseUserEvent):
    object: user_event_objects.UsersTypingStateObject


class DialogResetFlags(BaseUserEvent):
    object: user_event_objects.DialogResetFlagsObject


class DialogFlagsReplace(BaseUserEvent):
    object: user_event_objects.DialogFlagsReplaceObject


class DialogSetFlags(BaseUserEvent):
    object: user_event_objects.DialogSetFlagsObject


class FriendOnline(BaseUserEvent):
    object: user_event_objects.FriendOnlineObject


class FriendOffline(BaseUserEvent):
    object: user_event_objects.FriendOfflineObject


class FriendAction(BaseUserEvent):
    object: user_event_objects.FriendActionObject


class Counter(BaseUserEvent):
    object: user_event_objects.CounterObject


class Call(BaseUserEvent):
    object: user_event_objects.CallObject


class NotificationsSettingsChanged(BaseUserEvent):
    object: user_event_objects.NotificationsSettingsChangedObject


class InRead(BaseUserEvent):
    object: user_event_objects.InReadObject


class OutRead(BaseUserEvent):
    object: user_event_objects.OutReadObject


class CreateFolder(BaseUserEvent):
    object: user_event_objects.CreateFolderObject


class DeleteFolder(BaseUserEvent):
    object: user_event_objects.DeleteFolderObject


class RenameFolder(BaseUserEvent):
    object: user_event_objects.RenameFolderObject


class AddConversationsToFolder(BaseUserEvent):
    object: user_event_objects.AddConversationsToFolderObject


class RemoveConversationsFromFolder(BaseUserEvent):
    object: user_event_objects.RemoveConversationsFromFolderObject


class ChangeFolderOrder(BaseUserEvent):
    object: user_event_objects.ChangeFolderOrderObject


class CounterUnreadDialogsInFolders(BaseUserEvent):
    object: list[user_event_objects.CounterUnreadDialogsInFoldersObject]


__all__ = (
    "AddConversationsToFolder",
    "BaseUserEvent",
    "Call",
    "ChangeFolderOrder",
    "ChatEdit",
    "ChatInfoEdit",
    "ChatTypingState",
    "ChatVoiceMessageStates",
    "Counter",
    "CounterUnreadDialogsInFolders",
    "CreateFolder",
    "DeleteFolder",
    "DialogTypingState",
    "FriendOffline",
    "FriendOnline",
    "InRead",
    "MessageFlagsReplace",
    "MessageSetFlags",
    "MessageResetFlags",
    "MessageEdit",
    "MessageNew",
    "MessagesDelete",
    "MessagesRestore",
    "NotificationsSettingsChanged",
    "OutRead",
    "RawUserEvent",
    "RemoveConversationsFromFolder",
    "RenameFolder",
    "DialogFlagsReplace",
    "DialogResetFlags",
    "DialogSetFlags",
    "UsersTypingState",
)



import enum
from datetime import datetime, timezone
from functools import cached_property
from typing import Any

import msgspec

JsonObject = dict[str, Any] | list[Any]
Attachments = JsonObject
ExtraValues = JsonObject


class BaseEventObject(msgspec.Struct, omit_defaults=True, array_like=True, dict=True):
    pass


class MessageObject(BaseEventObject):
    message_id: int | None = None
    flags: int | None = None
    peer_id: int | None = None
    timestamp: int | None = None
    text: str | None = None
    extra_values: ExtraValues | None = None
    attachments: Attachments | None = None
    random_id: int | None = None

    @cached_property
    def date(self) -> datetime | None:
        if self.timestamp is None:
            return None
        return datetime.fromtimestamp(timestamp=self.timestamp, tz=timezone.utc)

    @property
    def message(self) -> str | None:
        if self.text is None:
            return None
        return self.text.replace("<br>", "\n").replace("&lt;", "<").replace("&gt;", ">").replace("&quot;", '"').replace("&amp;", "&")


class MessageFlagsReplaceObject(MessageObject):
    pass


class MessageSetFlagsObject(MessageObject):
    pass


class MessageResetFlagsObject(MessageObject):
    pass


class MessageNewObject(MessageObject):
    pass


class MessageEditObject(MessageObject):
    pass


class InReadObject(BaseEventObject):
    peer_id: int | None = None
    local_id: int | None = None


class OutReadObject(InReadObject):
    pass


class FriendOnlineObject(BaseEventObject):
    user_id: int | None = None
    extra: int = 0
    timestamp: int | None = None


class FriendOfflineObject(BaseEventObject):
    user_id: int | None = None
    flags: int = 0
    timestamp: int | None = None


class DialogResetFlagsObject(BaseEventObject):
    peer_id: int | None = None
    mask: int | None = None


class DialogFlagsReplaceObject(BaseEventObject):
    peer_id: int | None = None
    flags: int | None = None


class DialogSetFlagsObject(DialogResetFlagsObject):
    pass


class MessagesDeleteObject(InReadObject):
    pass


class MessagesRestoreObject(InReadObject):
    pass


class ChangeConversationParamsObject(BaseEventObject):
    chat_id: int | None = None
    self: int | None = None


class DialogTypingStateObject(BaseEventObject):
    user_id: int | None = None
    flags: int | None = None


class ConversationTypingStateObject(BaseEventObject):
    user_id: int | None = None
    chat_id: int | None = None


class UsersTypingStateObject(BaseEventObject):
    user_ids: list[int] | None = None
    peer_id: int | None = None
    total_count: int | None = None
    ts: int | None = None


class CallObject(BaseEventObject):
    user_id: int | None = None
    call_id: int | None = None


class CounterObject(BaseEventObject):
    count: int | None = None


class NotificationsSettingsChangedObject(BaseEventObject):
    peer_id: int | None = None
    sound: int | None = None
    disabled_until: int | None = None


class ChatInfoEditObject(BaseEventObject):
    type_id: int | None = None
    peer_id: int | None = None
    info: str | int | None = None


class ChatVoiceMessageStatesObject(BaseEventObject):
    user_ids: list[int] | None = None
    peer_id: int | None = None
    total_count: int | None = None
    ts: int | None = None


class ChatEditObject(BaseEventObject):
    chat_id: int | None = None
    self: int | None = None


class ActionType(enum.IntEnum):
    ACCEPT = 2
    REMOVE_OR_CANCEL = 3


class FriendActionObject(BaseEventObject):
    action_type: ActionType | None = None
    user_id: int | None = None


class CreateFolderObject(BaseEventObject):
    folder_id: int | None = None
    folder_name: str | None = None
    random_id: int | None = None


class DeleteFolderObject(BaseEventObject):
    folder_id: int | None = None


class RenameFolderObject(BaseEventObject):
    folder_id: int | None = None
    new_folder_name: str | None = None


class AddConversationsToFolderObject(BaseEventObject):
    folder_id: int | None = None
    peer_ids: list[int] | None = None


class RemoveConversationsFromFolderObject(BaseEventObject):
    folder_id: int | None = None
    peer_ids: list[int] | None = None


class ChangeFolderOrderObject(BaseEventObject):
    folder_ids: list[int] | None = None


class CounterUnreadDialogsInFoldersObject(BaseEventObject):
    folder_id: int | None = None
    unread_count: int | None = None
    unread_unmuted_count: int | None = None


__all__ = (
    "AddConversationsToFolderObject",
    "CallObject",
    "ChangeConversationParamsObject",
    "ChangeFolderOrderObject",
    "ChatEditObject",
    "ChatInfoEditObject",
    "ChatVoiceMessageStatesObject",
    "ConversationTypingStateObject",
    "CounterObject",
    "CounterUnreadDialogsInFoldersObject",
    "CreateFolderObject",
    "DeleteFolderObject",
    "DialogTypingStateObject",
    "FriendActionObject",
    "FriendOfflineObject",
    "FriendOnlineObject",
    "InReadObject",
    "MessageEditObject",
    "MessageNewObject",
    "MessageObject",
    "MessageResetFlagsObject",
    "MessageFlagsReplaceObject",
    "MessageSetFlagsObject",
    "MessagesDeleteObject",
    "MessagesRestoreObject",
    "NotificationsSettingsChangedObject",
    "OutReadObject",
    "RemoveConversationsFromFolderObject",
    "RenameFolderObject",
    "DialogSetFlagsObject",
    "DialogFlagsReplaceObject",
    "DialogResetFlagsObject",
    "MessageFlagsReplaceObject",
    "MessageSetFlagsObject",
    "MessageResetFlagsObject",
    "UsersTypingStateObject",
)



from vkbottle_types.methods.account import *
from vkbottle_types.methods.ads import *
from vkbottle_types.methods.app_widgets import *
from vkbottle_types.methods.apps import *
from vkbottle_types.methods.auth import *
from vkbottle_types.methods.base_category import *
from vkbottle_types.methods.board import *
from vkbottle_types.methods.bugtracker import *
from vkbottle_types.methods.database import *
from vkbottle_types.methods.docs import *
from vkbottle_types.methods.donut import *
from vkbottle_types.methods.downloaded_games import *
from vkbottle_types.methods.fave import *
from vkbottle_types.methods.friends import *
from vkbottle_types.methods.gifts import *
from vkbottle_types.methods.groups import *
from vkbottle_types.methods.lead_forms import *
from vkbottle_types.methods.likes import *
from vkbottle_types.methods.market import *
from vkbottle_types.methods.messages import *
from vkbottle_types.methods.newsfeed import *
from vkbottle_types.methods.notes import *
from vkbottle_types.methods.notifications import *
from vkbottle_types.methods.orders import *
from vkbottle_types.methods.pages import *
from vkbottle_types.methods.photos import *
from vkbottle_types.methods.podcasts import *
from vkbottle_types.methods.polls import *
from vkbottle_types.methods.pretty_cards import *
from vkbottle_types.methods.search import *
from vkbottle_types.methods.secure import *
from vkbottle_types.methods.stats import *
from vkbottle_types.methods.status import *
from vkbottle_types.methods.storage import *
from vkbottle_types.methods.store import *
from vkbottle_types.methods.stories import *
from vkbottle_types.methods.streaming import *
from vkbottle_types.methods.translations import *
from vkbottle_types.methods.users import *
from vkbottle_types.methods.utils import *
from vkbottle_types.methods.video import *
from vkbottle_types.methods.wall import *
from vkbottle_types.methods.widgets import *



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import StatsPeriod, StatsWallpostStat
from vkbottle_types.responses.base import (
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.stats import *  # noqa: F401,F403  # type: ignore


class StatsCategory(BaseCategory):
    async def get(
        self,
        app_id: int | None = None,
        extended: bool | None = None,
        filters: list[str] | None = None,
        group_id: int | None = None,
        interval: str | None = None,
        intervals_count: int | None = None,
        stats_groups: list[str] | None = None,
        timestamp_from: float | None = None,
        timestamp_to: float | None = None,
        **kwargs: typing.Any,
    ) -> list[StatsPeriod]:
        """Method `stats.get()`

        :param app_id: Application ID.
        :param extended:
        :param filters:
        :param group_id: Community ID.
        :param interval:
        :param intervals_count:
        :param stats_groups:
        :param timestamp_from:
        :param timestamp_to:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("stats.get", params)
        model = StatsGetResponse
        return model(**response).response

    async def get_post_reach(
        self,
        owner_id: int,
        post_ids: list[int],
        **kwargs: typing.Any,
    ) -> list[StatsWallpostStat]:
        """Method `stats.getPostReach()`

        :param owner_id: post owner community id. Specify with "-" sign.
        :param post_ids: wall posts id
        """

        params = self.get_set_params(locals())
        response = await self.api.request("stats.getPostReach", params)
        model = StatsGetPostReachResponse
        return model(**response).response

    async def track_visitor(
        self,
        type: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `stats.trackVisitor()`

        :param type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("stats.trackVisitor", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("StatsCategory",)



from vkbottle_types.codegen.responses.wall import *  # noqa: F403,F401  # type: ignore


class WallGetByIdExtendedResponseModel(WallGetByIdExtendedResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] = Field()  # type: ignore


class WallGetByIdExtendedResponse(WallGetByIdExtendedResponse):  # type: ignore[no-redef]
    response: "WallGetByIdExtendedResponseModel" = Field()


class WallGetByIdResponseModel(WallGetByIdResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] | None = Field(  # type: ignore
        default=None,
    )


class WallGetByIdResponse(WallGetByIdResponse):  # type: ignore[no-redef]
    response: "WallGetByIdResponseModel" = Field()


class WallGetExtendedResponseModel(WallGetExtendedResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] = Field()  # type: ignore


class WallGetExtendedResponse(WallGetExtendedResponse):  # type: ignore[no-redef]
    response: "WallGetExtendedResponseModel" = Field()


class WallGetResponseModel(WallGetResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] = Field()  # type: ignore


class WallGetResponse(WallGetResponse):  # type: ignore[no-redef]
    response: "WallGetResponseModel" = Field()


class WallSearchExtendedResponseModel(WallSearchExtendedResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] = Field()  # type: ignore


class WallSearchExtendedResponse(WallSearchExtendedResponse):  # type: ignore[no-redef]
    response: "WallSearchExtendedResponseModel" = Field()


class WallSearchResponseModel(WallSearchResponseModel):  # type: ignore[no-redef]
    items: list["WallWallpostFull"] = Field()  # type: ignore


class WallSearchResponse(WallSearchResponse):  # type: ignore[no-redef]
    response: "WallSearchResponseModel" = Field()



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import AppWidgetsPhoto, AppWidgetsPhotos
from vkbottle_types.responses.app_widgets import *  # noqa: F401,F403  # type: ignore
from vkbottle_types.responses.base import (
    BaseOkResponse,
    BaseOkResponseModel,
)


class AppWidgetsCategory(BaseCategory):
    async def get_app_image_upload_server(
        self,
        image_type: str,
        **kwargs: typing.Any,
    ) -> AppWidgetsGetAppImageUploadServerResponseModel:
        """Method `appWidgets.getAppImageUploadServer()`

        :param image_type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.getAppImageUploadServer", params)
        model = AppWidgetsGetAppImageUploadServerResponse
        return model(**response).response

    async def get_app_images(
        self,
        count: int | None = None,
        image_type: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> "AppWidgetsPhotos":
        """Method `appWidgets.getAppImages()`

        :param count: Maximum count of results.
        :param image_type:
        :param offset: Offset needed to return a specific subset of images.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.getAppImages", params)
        model = AppWidgetsGetAppImagesResponse
        return model(**response).response

    async def get_group_image_upload_server(
        self,
        image_type: str,
        **kwargs: typing.Any,
    ) -> AppWidgetsGetGroupImageUploadServerResponseModel:
        """Method `appWidgets.getGroupImageUploadServer()`

        :param image_type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.getGroupImageUploadServer", params)
        model = AppWidgetsGetGroupImageUploadServerResponse
        return model(**response).response

    async def get_group_images(
        self,
        count: int | None = None,
        image_type: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> "AppWidgetsPhotos":
        """Method `appWidgets.getGroupImages()`

        :param count: Maximum count of results.
        :param image_type:
        :param offset: Offset needed to return a specific subset of images.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.getGroupImages", params)
        model = AppWidgetsGetGroupImagesResponse
        return model(**response).response

    async def get_images_by_id(
        self,
        images: list[str],
        **kwargs: typing.Any,
    ) -> list[AppWidgetsPhoto]:
        """Method `appWidgets.getImagesById()`

        :param images: List of images IDs
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.getImagesById", params)
        model = AppWidgetsGetImagesByIdResponse
        return model(**response).response

    async def save_app_image(
        self,
        hash: str,
        image: str,
        **kwargs: typing.Any,
    ) -> "AppWidgetsPhoto":
        """Method `appWidgets.saveAppImage()`

        :param hash: Parameter returned when photo is uploaded to server
        :param image: Parameter returned when photo is uploaded to server
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.saveAppImage", params)
        model = AppWidgetsSaveAppImageResponse
        return model(**response).response

    async def save_group_image(
        self,
        hash: str,
        image: str,
        **kwargs: typing.Any,
    ) -> "AppWidgetsPhoto":
        """Method `appWidgets.saveGroupImage()`

        :param hash: Parameter returned when photo is uploaded to server
        :param image: Parameter returned when photo is uploaded to server
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.saveGroupImage", params)
        model = AppWidgetsSaveGroupImageResponse
        return model(**response).response

    async def update(
        self,
        code: str,
        type: str,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `appWidgets.update()`

        :param code:
        :param type:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("appWidgets.update", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("AppWidgetsCategory",)



import typing

from typing_extensions import Literal

from vkbottle_types.codegen.methods.video import VideoCategory as _VideoCategory  # type: ignore
from vkbottle_types.codegen.responses.base import BaseOkResponse, BaseOkResponseModel
from vkbottle_types.codegen.responses.video import VideoChangeVideoAlbumsResponse

if typing.TYPE_CHECKING:
    from vkbottle import ABCAPI  # type: ignore


class VideoCategory(_VideoCategory):  # type: ignore
    def __init__(self, api: "ABCAPI") -> None:
        super().__init__(api)

    @typing.overload  # type: ignore
    async def add_to_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: int | None = ...,
        album_ids: Literal[None] | None = None,
        **kwargs,
    ) -> BaseOkResponseModel: ...

    @typing.overload
    async def add_to_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: Literal[None] | None = None,
        album_ids: list[int] | None = ...,
        **kwargs,
    ) -> list[int]: ...

    async def add_to_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: int | None = None,
        album_ids: list[int] | None = None,
        **kwargs,
    ) -> BaseOkResponseModel | list[int]:
        """video.addToAlbum method

        :param owner_id:
        :param video_id:
        :param target_id:
        :param album_id:
        :param album_ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("video.addToAlbum", params)
        model = self.get_model(
            ((("album_ids",), VideoChangeVideoAlbumsResponse),),
            default=BaseOkResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload  # type: ignore
    async def remove_from_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: int | None = ...,
        album_ids: Literal[None] | None = None,
        **kwargs,
    ) -> BaseOkResponseModel: ...

    @typing.overload
    async def remove_from_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: Literal[None] | None = None,
        album_ids: list[int] | None = ...,
        **kwargs,
    ) -> list[int]: ...

    async def remove_from_album(
        self,
        owner_id: int,
        video_id: int,
        target_id: int | None = None,
        album_id: int | None = None,
        album_ids: list[int] | None = None,
        **kwargs,
    ) -> BaseOkResponseModel | list[int]:
        """video.removeFromAlbum method

        :param owner_id:
        :param video_id:
        :param target_id:
        :param album_id:
        :param album_ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("video.removeFromAlbum", params)
        model = self.get_model(
            ((("album_ids",), VideoChangeVideoAlbumsResponse),),
            default=BaseOkResponse,
            params=params,
        )
        return model(**response).response


__all__ = ("VideoCategory",)



import typing

from vkbottle_types.codegen.methods.utils import UtilsCategory as _UtilsCategory  # type: ignore
from vkbottle_types.objects import UtilsDomainResolved
from vkbottle_types.responses import UtilsResolveScreenNameResponse  # noqa: F401,F403  # type: ignore


class UtilsCategory(_UtilsCategory):  # type: ignore
    async def resolve_screen_name(  # type: ignore
        self,
        screen_name: str,
        **kwargs: typing.Any,
    ) -> UtilsDomainResolved | list[typing.Any]:
        """Method `utils.resolveScreenName()`

        :param screen_name: Screen name of the user, community (e.g., 'apiclub,' 'andrew', or 'rules_of_war'), or application.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.resolveScreenName", params)
        model = UtilsResolveScreenNameResponse
        return model(**response).response


__all__ = ("UtilsCategory",)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import BaseStickerNew, StoreProduct, StoreStickersKeyword
from vkbottle_types.responses.base_response import BaseResponse


class StoreGetFavoriteStickersResponseModel(BaseModel):
    count: int = Field()
    items: list["BaseStickerNew"] = Field()


class StoreGetFavoriteStickersResponse(BaseResponse):
    response: "StoreGetFavoriteStickersResponseModel" = Field()


class StoreGetProductsResponseModel(BaseModel):
    items: list["StoreProduct"] = Field()
    count: int = Field()


class StoreGetProductsResponse(BaseResponse):
    response: "StoreGetProductsResponseModel" = Field()


class StoreGetStickersKeywordsResponseModel(BaseModel):
    count: int = Field()
    dictionary: list["StoreStickersKeyword"] = Field()
    chunks_count: int | None = Field(
        default=None,
    )
    chunks_hash: str | None = Field(
        default=None,
    )


class StoreGetStickersKeywordsResponse(BaseResponse):
    response: "StoreGetStickersKeywordsResponseModel" = Field()



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import GroupsGroupFull, StoriesFeedItem, StoriesStory, StoriesStoryStats, StoriesViewersItem, UsersUser, UsersUserFull
from vkbottle_types.responses.base_response import BaseResponse


class StoriesGetBannedExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list[int] = Field()
    profiles: list["UsersUserFull"] = Field()
    groups: list["GroupsGroupFull"] = Field()


class StoriesGetBannedExtendedResponse(BaseResponse):
    response: "StoriesGetBannedExtendedResponseModel" = Field()


class StoriesGetBannedResponseModel(BaseModel):
    count: int = Field()
    items: list[int] = Field()


class StoriesGetBannedResponse(BaseResponse):
    response: "StoriesGetBannedResponseModel" = Field()


class StoriesGetByIdExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list["StoriesStory"] = Field()
    profiles: list["UsersUserFull"] = Field()
    groups: list["GroupsGroupFull"] = Field()


class StoriesGetByIdExtendedResponse(BaseResponse):
    response: "StoriesGetByIdExtendedResponseModel" = Field()


class StoriesGetPhotoUploadServerResponseModel(BaseModel):
    upload_url: str = Field()
    user_ids: list[int] = Field()


class StoriesGetPhotoUploadServerResponse(BaseResponse):
    response: "StoriesGetPhotoUploadServerResponseModel" = Field()


class StoriesGetStatsResponse(BaseResponse):
    response: "StoriesStoryStats" = Field()


class StoriesGetVideoUploadServerResponseModel(BaseModel):
    upload_url: str = Field()
    user_ids: list[int] = Field()


class StoriesGetVideoUploadServerResponse(BaseResponse):
    response: "StoriesGetVideoUploadServerResponseModel" = Field()


class StoriesGetViewersExtendedV5115ResponseModel(BaseModel):
    count: int = Field()
    items: list["StoriesViewersItem"] = Field()
    hidden_reason: str | None = Field(
        default=None,
    )
    next_from: str | None = Field(
        default=None,
    )


class StoriesGetViewersExtendedV5115Response(BaseResponse):
    response: "StoriesGetViewersExtendedV5115ResponseModel" = Field()


class StoriesGetV5113ResponseModel(BaseModel):
    count: int = Field()
    items: list["StoriesFeedItem"] = Field()
    profiles: list["UsersUserFull"] | None = Field(
        default=None,
    )
    groups: list["GroupsGroupFull"] | None = Field(
        default=None,
    )
    need_upload_screen: bool | None = Field(
        default=None,
    )
    track_code: str | None = Field(
        default=None,
    )
    next_from: str | None = Field(
        default=None,
    )


class StoriesGetV5113Response(BaseResponse):
    response: "StoriesGetV5113ResponseModel" = Field()


class StoriesSaveResponseModel(BaseModel):
    count: int = Field()
    items: list["StoriesStory"] = Field()
    profiles: list["UsersUser"] | None = Field(
        default=None,
    )
    groups: list["GroupsGroupFull"] | None = Field(
        default=None,
    )


class StoriesSaveResponse(BaseResponse):
    response: "StoriesSaveResponseModel" = Field()


class StoriesUploadResponseModel(BaseModel):
    upload_result: str | None = Field(
        default=None,
    )


class StoriesUploadResponse(BaseResponse):
    response: "StoriesUploadResponseModel" = Field()



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import UsersFields
from vkbottle_types.responses.widgets import *  # noqa: F401,F403  # type: ignore


class WidgetsCategory(BaseCategory):
    async def get_comments(
        self,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        offset: int | None = None,
        order: str | None = None,
        page_id: str | None = None,
        url: str | None = None,
        widget_api_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WidgetsGetCommentsResponseModel:
        """Method `widgets.getComments()`

        :param count:
        :param fields:
        :param offset:
        :param order:
        :param page_id:
        :param url:
        :param widget_api_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("widgets.getComments", params)
        model = WidgetsGetCommentsResponse
        return model(**response).response

    async def get_pages(
        self,
        count: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        period: str | None = None,
        widget_api_id: int | None = None,
        **kwargs: typing.Any,
    ) -> WidgetsGetPagesResponseModel:
        """Method `widgets.getPages()`

        :param count:
        :param offset:
        :param order:
        :param period:
        :param widget_api_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("widgets.getPages", params)
        model = WidgetsGetPagesResponse
        return model(**response).response


__all__ = ("WidgetsCategory",)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import (
    UtilsDomainResolved,
    UtilsLastShortenedLink,
    UtilsLinkChecked,
    UtilsLinkStats,
    UtilsLinkStatsExtended,
    UtilsShortLink,
)
from vkbottle_types.responses.base_response import BaseResponse


class UtilsCheckLinkResponse(BaseResponse):
    response: "UtilsLinkChecked" = Field()


class UtilsGetLastShortenedLinksResponseModel(BaseModel):
    count: int = Field()
    items: list["UtilsLastShortenedLink"] = Field()


class UtilsGetLastShortenedLinksResponse(BaseResponse):
    response: "UtilsGetLastShortenedLinksResponseModel" = Field()


class UtilsGetLinkStatsExtendedResponse(BaseResponse):
    response: "UtilsLinkStatsExtended" = Field()


class UtilsGetLinkStatsResponse(BaseResponse):
    response: "UtilsLinkStats" = Field()


class UtilsGetServerTimeResponse(BaseResponse):
    response: int = Field()


class UtilsGetShortLinkResponse(BaseResponse):
    response: "UtilsShortLink" = Field()


class UtilsResolveScreenNameResponse(BaseResponse):
    response: "UtilsDomainResolved" = Field()



import itertools
import keyword

BUILTIN_TYPES = frozenset(("list", "dict", "str", "int", "float", "bool", "set", "frozenset", "tuple", "bytes"))


def camelcase(s: str) -> str:
    return "".join(word[0].upper() + word[1:] for word in s.split("_"))


def snake_case(s: str) -> str:
    """Converts string into snake case.

    >>> snake_case("camelCase")
    >>> 'camel_case'
    >>> snake_case("PascalCase")
    >>> 'pascal_case'
    >>> snake_case("getSafeURL")
    >>> 'get_safe_url'
    """

    new_s = ""
    for first, second in itertools.zip_longest(s, s[1:], fillvalue=""):
        if first.islower() and second.isupper():
            new_s += first + "_"
        else:
            new_s += first.lower()
    return new_s


def makesafe(s: str) -> str:
    """Transforms arbitrary string into python-declarable name."""

    if keyword.iskeyword(s) or s == "model" or s in BUILTIN_TYPES:
        s = s + "_"
    elif s[0].isdigit():
        s = "f__" + s
    s = s.replace(" ", "_")
    if s.isupper():
        return s
    return snake_case(s)


def makesafe_method_name(s: str) -> str:
    if keyword.iskeyword(s):
        return s + "_"
    if s in ("dict", "list"):
        return "get_" + s
    return s


def instring(s: str) -> str:
    return s.replace("'", "\\'").replace('"', '\\"')


__all__ = ("camelcase", "snake_case", "makesafe", "instring", "makesafe_method_name")



import typing

from vkbottle_types.codegen.methods.ads import AdsCategory as _AdsCategory  # type: ignore
from vkbottle_types.responses.ads import (
    AdsGetSuggestionsCitiesResponse,
    AdsGetSuggestionsRegionsResponse,
    AdsGetSuggestionsResponse,
    AdsGetSuggestionsSchoolsResponse,
    AdsTargSuggestions,
    AdsTargSuggestionsCities,
    AdsTargSuggestionsRegions,
    AdsTargSuggestionsSchools,
)


class AdsCategory(_AdsCategory):  # type: ignore
    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        *,
        q: typing.Literal["regions"],
        country: int,
        ids: list[str] | None = None,
        lang: str | None = None,
    ) -> list[AdsTargSuggestionsRegions]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        *,
        q: typing.Literal["schools"],
        cities: list[str],
        country: int | None = None,
        ids: list[str] | None = None,
        lang: str | None = None,
    ) -> list[AdsTargSuggestionsSchools]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        *,
        country: int | None = None,
        ids: list[str] | None = None,
        lang: str | None = None,
        q: str | None = None,
    ) -> list[AdsTargSuggestions]: ...

    @typing.overload
    async def get_suggestions(
        self,
        section: str,
        *,
        cities: list[str],
        country: int | None = None,
        ids: list[str] | None = None,
        lang: str | None = None,
        q: str | None = None,
    ) -> list[AdsTargSuggestionsCities]: ...

    async def get_suggestions(
        self,
        section: str,
        *,
        cities: list[str] | None = None,
        country: int | None = None,
        ids: list[str] | None = None,
        lang: str | None = None,
        q: str | None = None,
        **kwargs: typing.Any,
    ) -> list[AdsTargSuggestionsSchools] | list[AdsTargSuggestionsCities] | list[AdsTargSuggestionsRegions] | list[AdsTargSuggestions]:
        """Method `ads.getSuggestions()`

        :param section: Section, suggestions are retrieved in. Available values: *countries - request of a list of countries. If q is not set or blank, a short list of countries is shown. Otherwise, a full list of countries is shown. *regions - requested list of regions. 'country' parameter is required. *cities - requested list of cities. 'country' parameter is required. *districts - requested list of districts. 'cities' parameter is required. *stations - requested list of subway stations. 'cities' parameter is required. *streets - requested list of streets. 'cities' parameter is required. *schools - requested list of educational organizations. 'cities' parameter is required. *interests - requested list of interests. *positions - requested list of positions (professions). *group_types - requested list of group types. *religions - requested list of religious commitments. *browsers - requested list of browsers and mobile devices.
        :param cities: IDs of cities where objects are searched in, separated with a comma.
        :param country: ID of the country objects are searched in.
        :param ids: Objects IDs separated by commas. If the parameter is passed, 'q, country, cities' should not be passed.
        :param lang: Language of the returned string values. Supported languages: *ru - Russian,, *ua - Ukrainian,, *en - English.
        :param q: Filter-line of the request (for countries, regions, cities, streets, schools, interests, positions).
        """

        params = self.get_set_params(locals())  # type: ignore
        response = await self.api.request("ads.getSuggestions", params)  # type: ignore
        model = self.get_model(  # type: ignore
            (
                dict(q=dict(regions=AdsGetSuggestionsRegionsResponse, schools=AdsGetSuggestionsSchoolsResponse)),
                (("cities",), AdsGetSuggestionsCitiesResponse),
            ),
            default=AdsGetSuggestionsResponse,
            params=params,
        )
        return model(**response).response



import typing

from vkbottle_types.codegen.methods.newsfeed import NewsfeedCategory as _NewsfeedCategory  # type: ignore
from vkbottle_types.objects import BaseUserGroupFields
from vkbottle_types.responses.newsfeed import (
    NewsfeedSearchExtendedResponse,
    NewsfeedSearchExtendedResponseModel,
    NewsfeedSearchExtendedStrictResponse,
    NewsfeedSearchExtendedStrictResponseModel,
    NewsfeedSearchResponse,
    NewsfeedSearchResponseModel,
    NewsfeedSearchStrictResponse,
    NewsfeedSearchStrictResponseModel,
)


class NewsfeedCategory(_NewsfeedCategory):  # type: ignore
    @typing.overload  # type: ignore
    async def search(
        self,
        *,
        strict: typing.Literal[True],
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
    ) -> NewsfeedSearchStrictResponseModel: ...

    @typing.overload
    async def search(
        self,
        *,
        extended: typing.Literal[True],
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
    ) -> NewsfeedSearchExtendedResponseModel: ...

    @typing.overload
    async def search(
        self,
        *,
        strict: typing.Literal[True],
        extended: typing.Literal[True],
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
    ) -> NewsfeedSearchExtendedStrictResponseModel: ...

    @typing.overload
    async def search(
        self,
        *,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
    ) -> NewsfeedSearchResponseModel: ...

    async def search(
        self,
        *,
        extended: bool | None = None,
        strict: bool | None = None,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> (
        NewsfeedSearchResponseModel
        | NewsfeedSearchExtendedResponseModel
        | NewsfeedSearchExtendedStrictResponseModel
        | NewsfeedSearchStrictResponseModel
    ):
        """Method `newsfeed.search()`

        :param extended: '1' - to return additional information about the user or community that placed the post.
        :param count: Number of posts to return.
        :param end_time: Latest timestamp (in Unix time) of a news item to return. By default, the current time.
        :param fields: Additional fields of [vk.ru/dev/fields|profiles] and [vk.ru/dev/fields_groups|communities] to return.
        :param latitude: Geographical latitude point (in degrees, -90 to 90) within which to search.
        :param longitude: Geographical longitude point (in degrees, -180 to 180) within which to search.
        :param q: Search query string (e.g., 'New Year').
        :param start_from:
        :param start_time: Earliest timestamp (in Unix time) of a news item to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.search", params)
        model = self.get_model(
            (
                (("strict",), NewsfeedSearchStrictResponse),
                (("extended",), NewsfeedSearchExtendedResponse),
                (("extended", "strict"), NewsfeedSearchExtendedStrictResponse),
            ),
            default=NewsfeedSearchResponse,
            params=params,
        )
        return model(**response).response



import typing

from vkbottle_types.base_model import BaseModel, Field

Response = typing.TypeVar("Response")


class ExecuteResponse(BaseModel, typing.Generic[Response]):
    response: Response = Field()


__all__ = ("ExecuteResponse",)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import BaseUserGroupFields, NewsfeedCommentsFilters, NewsfeedNewsfeedItemType, UsersFields
from vkbottle_types.responses.base import (
    BaseBoolResponse,
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.newsfeed import *  # noqa: F401,F403  # type: ignore


class NewsfeedCategory(BaseCategory):
    async def add_ban(
        self,
        group_ids: list[int] | None = None,
        user_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> bool:
        """Method `newsfeed.addBan()`

        :param group_ids:
        :param user_ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.addBan", params)
        model = BaseBoolResponse
        return model(**response).response

    async def delete_ban(
        self,
        group_ids: list[int] | None = None,
        user_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `newsfeed.deleteBan()`

        :param group_ids:
        :param user_ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.deleteBan", params)
        model = BaseOkResponse
        return model(**response).response

    async def delete_list(
        self,
        list_id: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `newsfeed.deleteList()`

        :param list_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.deleteList", params)
        model = BaseOkResponse
        return model(**response).response

    async def get(
        self,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filters: list[NewsfeedNewsfeedItemType] | None = None,
        max_photos: int | None = None,
        return_banned: bool | None = None,
        section: str | None = None,
        source_ids: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGenericResponseModel:
        """Method `newsfeed.get()`

        :param count: Number of news items to return (default 50, maximum 100). For auto feed, you can use the 'new_offset' parameter returned by this method.
        :param end_time: Latest timestamp (in Unix time) of a news item to return. By default, the current time.
        :param fields: Additional fields of [vk.com/dev/fields|profiles] and [vk.com/dev/fields_groups|communities] to return.
        :param filters: Filters to apply: 'post' - new wall posts, 'photo' - new photos, 'photo_tag' - new photo tags, 'wall_photo' - new wall photos, 'friend' - new friends
        :param max_photos: Maximum number of photos to return. By default, '5'.
        :param return_banned: '1' - to return news items from banned sources
        :param section:
        :param source_ids: Sources to obtain news from, separated by commas. User IDs can be specified in formats '' or 'u' , where '' is the user's friend ID. Community IDs can be specified in formats '-' or 'g' , where '' is the community ID. If the parameter is not set, all of the user's friends and communities are returned, except for banned sources, which can be obtained with the [vk.com/dev/newsfeed.getBanned|newsfeed.getBanned] method.
        :param start_from: identifier required to get the next page of results. Value for this parameter is returned in 'next_from' field in a reply.
        :param start_time: Earliest timestamp (in Unix time) of a news item to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.get", params)
        model = NewsfeedGenericResponse
        return model(**response).response

    @typing.overload
    async def get_banned(
        self,
        extended: typing.Literal[True],
        fields: list[UsersFields] | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetBannedExtendedResponseModel: ...

    @typing.overload
    async def get_banned(
        self,
        extended: typing.Literal[False] | None = None,
        fields: list[UsersFields] | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetBannedResponseModel: ...

    async def get_banned(
        self,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        name_case: str | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetBannedResponseModel | NewsfeedGetBannedExtendedResponseModel:
        """Method `newsfeed.getBanned()`

        :param extended: '1' - return extra information about users and communities
        :param fields: Profile fields to return.
        :param name_case: Case for declension of user name and surname: 'nom' - nominative (default), 'gen' - genitive , 'dat' - dative, 'acc' - accusative , 'ins' - instrumental , 'abl' - prepositional
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getBanned", params)
        model = self.get_model(
            ((("extended",), NewsfeedGetBannedExtendedResponse),),
            default=NewsfeedGetBannedResponse,
            params=params,
        )
        return model(**response).response

    async def get_comments(
        self,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        filters: list[NewsfeedCommentsFilters] | None = None,
        last_comments_count: int | None = None,
        reposts: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetCommentsResponseModel:
        """Method `newsfeed.getComments()`

        :param count: Number of comments to return. For auto feed, you can use the 'new_offset' parameter returned by this method.
        :param end_time: Latest timestamp (in Unix time) of a comment to return. By default, the current time.
        :param fields: Additional fields of [vk.com/dev/fields|profiles] and [vk.com/dev/fields_groups|communities] to return.
        :param filters: Filters to apply: 'post' - new comments on wall posts, 'photo' - new comments on photos, 'video' - new comments on videos, 'topic' - new comments on discussions, 'note' - new comments on notes,
        :param last_comments_count:
        :param reposts: Object ID, comments on repost of which shall be returned, e.g. 'wall1_45486'. (If the parameter is set, the 'filters' parameter is optional.),
        :param start_from: Identificator needed to return the next page with results. Value for this parameter returns in 'next_from' field.
        :param start_time: Earliest timestamp (in Unix time) of a comment to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getComments", params)
        model = NewsfeedGetCommentsResponse
        return model(**response).response

    @typing.overload
    async def get_lists(
        self,
        extended: typing.Literal[True],
        list_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetListsExtendedResponseModel: ...

    @typing.overload
    async def get_lists(
        self,
        extended: typing.Literal[False] | None = None,
        list_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetListsResponseModel: ...

    async def get_lists(
        self,
        extended: bool | None = None,
        list_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetListsExtendedResponseModel | NewsfeedGetListsResponseModel:
        """Method `newsfeed.getLists()`

        :param extended: Return additional list info
        :param list_ids: numeric list identifiers.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getLists", params)
        model = self.get_model(
            ((("extended",), NewsfeedGetListsExtendedResponse),),
            default=NewsfeedGetListsResponse,
            params=params,
        )
        return model(**response).response

    async def get_mentions(
        self,
        count: int | None = None,
        end_time: int | None = None,
        offset: int | None = None,
        owner_id: int | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetMentionsResponseModel:
        """Method `newsfeed.getMentions()`

        :param count: Number of posts to return.
        :param end_time: Latest timestamp (in Unix time) of a post to return. By default, the current time.
        :param offset: Offset needed to return a specific subset of posts.
        :param owner_id: Owner ID.
        :param start_time: Earliest timestamp (in Unix time) of a post to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getMentions", params)
        model = NewsfeedGetMentionsResponse
        return model(**response).response

    async def get_recommended(
        self,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        max_photos: int | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGenericResponseModel:
        """Method `newsfeed.getRecommended()`

        :param count: Number of news items to return.
        :param end_time: Latest timestamp (in Unix time) of a news item to return. By default, the current time.
        :param fields: Additional fields of [vk.com/dev/fields|profiles] and [vk.com/dev/fields_groups|communities] to return.
        :param max_photos: Maximum number of photos to return. By default, '5'.
        :param start_from: 'new_from' value obtained in previous call.
        :param start_time: Earliest timestamp (in Unix time) of a news item to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getRecommended", params)
        model = NewsfeedGenericResponse
        return model(**response).response

    async def get_suggested_sources(
        self,
        count: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        offset: int | None = None,
        shuffle: bool | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedGetSuggestedSourcesResponseModel:
        """Method `newsfeed.getSuggestedSources()`

        :param count: amount of communities or users to return.
        :param fields: list of extra fields to be returned. See available fields for [vk.com/dev/fields|users] and [vk.com/dev/fields_groups|communities].
        :param offset: offset required to choose a particular subset of communities or users.
        :param shuffle: shuffle the returned list or not.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.getSuggestedSources", params)
        model = NewsfeedGetSuggestedSourcesResponse
        return model(**response).response

    async def ignore_item(
        self,
        type: str,
        item_id: int | None = None,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedIgnoreItemResponseModel:
        """Method `newsfeed.ignoreItem()`

        :param type: Item type. Possible values: *'wall' - post on the wall,, *'tag' - tag on a photo,, *'profilephoto' - profile photo,, *'video' - video,, *'audio' - audio.
        :param item_id: Item identifier
        :param owner_id: Item owner's identifier (user or community), "Note that community id must be negative. 'owner_id=1' - user , 'owner_id=-1' - community "
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.ignoreItem", params)
        model = NewsfeedIgnoreItemResponse
        return model(**response).response

    async def save_list(
        self,
        source_ids: list[int],
        title: str,
        list_id: int | None = None,
        no_reposts: bool | None = None,
        **kwargs: typing.Any,
    ) -> int:
        """Method `newsfeed.saveList()`

        :param source_ids: users and communities identifiers to be added to the list. Community identifiers must be negative numbers.
        :param title: list name.
        :param list_id: numeric list identifier (if not sent, will be set automatically).
        :param no_reposts: reposts display on and off ('1' is for off).
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.saveList", params)
        model = NewsfeedSaveListResponse
        return model(**response).response

    @typing.overload
    async def search(
        self,
        extended: typing.Literal[True],
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedSearchStrictResponseModel: ...

    @typing.overload
    async def search(
        self,
        extended: bool | None = None,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedSearchExtendedResponseModel: ...

    @typing.overload
    async def search(
        self,
        count: int | None = None,
        end_time: int | None = None,
        extended: bool | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedSearchExtendedStrictResponseModel: ...

    @typing.overload
    async def search(
        self,
        count: int | None = None,
        end_time: int | None = None,
        extended: bool | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedSearchResponseModel: ...

    async def search(
        self,
        extended: bool | None = None,
        count: int | None = None,
        end_time: int | None = None,
        fields: list[BaseUserGroupFields] | None = None,
        latitude: float | None = None,
        longitude: float | None = None,
        q: str | None = None,
        start_from: str | None = None,
        start_time: int | None = None,
        **kwargs: typing.Any,
    ) -> NewsfeedSearchStrictResponseModel | NewsfeedSearchExtendedResponseModel | NewsfeedSearchResponseModel:
        """Method `newsfeed.search()`

        :param extended: '1' - to return additional information about the user or community that placed the post.
        :param count: Number of posts to return.
        :param end_time: Latest timestamp (in Unix time) of a news item to return. By default, the current time.
        :param fields: Additional fields of [vk.com/dev/fields|profiles] and [vk.com/dev/fields_groups|communities] to return.
        :param latitude: Geographical latitude point (in degrees, -90 to 90) within which to search.
        :param longitude: Geographical longitude point (in degrees, -180 to 180) within which to search.
        :param q: Search query string (e.g., 'New Year').
        :param start_from:
        :param start_time: Earliest timestamp (in Unix time) of a news item to return. By default, 24 hours ago.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.search", params)
        model = self.get_model(
            (
                (("strict",), NewsfeedSearchStrictResponse),
                (("extended",), NewsfeedSearchStrictResponse),
                (("extended_strict",), NewsfeedSearchExtendedStrictResponse),
            ),
            default=NewsfeedSearchResponse,
            params=params,
        )
        return model(**response).response

    async def unignore_item(
        self,
        type: str,
        item_id: int | None = None,
        owner_id: int | None = None,
        track_code: str | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `newsfeed.unignoreItem()`

        :param type: Item type. Possible values: *'wall' - post on the wall,, *'tag' - tag on a photo,, *'profilephoto' - profile photo,, *'video' - video,, *'audio' - audio.
        :param item_id: Item identifier
        :param owner_id: Item owner's identifier (user or community), "Note that community id must be negative. 'owner_id=1' - user , 'owner_id=-1' - community "
        :param track_code: Track code of unignored item
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.unignoreItem", params)
        model = BaseOkResponse
        return model(**response).response

    async def unsubscribe(
        self,
        item_id: int,
        type: str,
        owner_id: int | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `newsfeed.unsubscribe()`

        :param item_id: Object ID.
        :param type: Type of object from which to unsubscribe: 'note' - note, 'photo' - photo, 'post' - post on user wall or community wall, 'topic' - topic, 'video' - video
        :param owner_id: Object owner ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("newsfeed.unsubscribe", params)
        model = BaseOkResponse
        return model(**response).response


__all__ = ("NewsfeedCategory",)



from typing_extensions import TypeAlias

from vkbottle_types.base_model import BaseEnumMeta, BaseModel, Field, StrEnum
from vkbottle_types.codegen.objects import *  # noqa: F403  # type: ignore


class MessagesMessageActionStatus(StrEnum, metaclass=BaseEnumMeta):  # type: ignore
    CHAT_PHOTO_UPDATE = "chat_photo_update"
    CHAT_PHOTO_REMOVE = "chat_photo_remove"
    CHAT_CREATE = "chat_create"
    CHAT_TITLE_UPDATE = "chat_title_update"
    CHAT_INVITE_USER = "chat_invite_user"
    CHAT_KICK_USER = "chat_kick_user"
    CHAT_PIN_MESSAGE = "chat_pin_message"
    CHAT_UNPIN_MESSAGE = "chat_unpin_message"
    CHAT_INVITE_USER_BY_LINK = "chat_invite_user_by_link"
    CHAT_INVITE_USER_BY_MESSAGE_REQUEST = "chat_invite_user_by_message_request"
    CHAT_SCREENSHOT = "chat_screenshot"
    CONVERSATION_STYLE_UPDATE = "conversation_style_update"


class MessagesMessageAttachmentType(StrEnum, metaclass=BaseEnumMeta):  # type: ignore
    PHOTO = "photo"
    AUDIO = "audio"
    VIDEO = "video"
    DOC = "doc"
    LINK = "link"
    MARKET = "market"
    MARKET_ALBUM = "market_album"
    GIFT = "gift"
    STICKER = "sticker"
    WALL = "wall"
    WALL_REPLY = "wall_reply"
    ARTICLE = "article"
    POLL = "poll"
    CALL = "call"
    GRAFFITI = "graffiti"
    AUDIO_MESSAGE = "audio_message"
    STORY = "story"
    GROUP_CALL_IN_PROGRESS = "group_call_in_progress"
    MINI_APP = "mini_app"
    VIDEO_PLAYLIST = "video_playlist"
    NARRATIVE = "narrative"


class VideoVideoType(StrEnum, metaclass=BaseEnumMeta):  # type: ignore
    INTERACTIVE = "interactive"
    VIDEO = "video"
    MUSIC_VIDEO = "music_video"
    MOVIE = "movie"
    VIDEO_MESSAGE = "video_message"
    LIVE = "live"
    SHORT_VIDEO = "short_video"
    STORY = "story"


class WallWallpostAttachmentType(StrEnum, metaclass=BaseEnumMeta):  # type: ignore
    """Attachment type"""

    PHOTO = "photo"
    PHOTOS_LIST = "photos_list"
    POSTED_PHOTO = "posted_photo"
    AUDIO = "audio"
    AUDIO_PLAYLIST = "audio_playlist"
    VIDEO = "video"
    DOC = "doc"
    LINK = "link"
    GRAFFITI = "graffiti"
    NOTE = "note"
    APP = "app"
    POLL = "poll"
    PAGE = "page"
    ALBUM = "album"
    MARKET_ALBUM = "market_album"
    MARKET = "market"
    EVENT = "event"
    DONUT_LINK = "donut_link"
    ARTICLE = "article"
    TEXTLIVE = "textlive"
    TEXTPOST = "textpost"
    TEXTPOST_PUBLISH = "textpost_publish"
    SITUATIONAL_THEME = "situational_theme"
    GROUP = "group"
    STICKER = "sticker"
    PODCAST = "podcast"
    PRETTY_CARDS = "pretty_cards"
    MINI_APP = "mini_app"
    CLIP = "clip"
    VIDEO_PLAYLIST = "video_playlist"


class PhotosPhoto(PhotosPhoto):  # type: ignore
    orig_photo: "PhotosPhotoSizes | None" = None


class PrettyCardsList(BaseModel):
    cards: "PrettyCardsPrettyCard | None" = None


class PollsPoll(PollsPoll):  # type: ignore[no-redef]
    anonymous: bool | None = None  # type: ignore[assignment]


class WallWallpostAttachment(WallWallpostAttachment):  # type: ignore
    mini_app: "AppsApp | None" = None
    pretty_cards: "PrettyCardsList | None" = None
    poll: "PollsPoll | None" = None


class PollsPollExtended(PollsPoll):  # type: ignore[no-redef]
    pass


class CallbackLikeAddRemoveObjectType(StrEnum, metaclass=BaseEnumMeta):  # type: ignore[no-redef]
    VIDEO = "video"
    PHOTO = "photo"
    POST = "post"
    COMMENT = "comment"
    NOTE = "note"
    TOPIC_COMMENT = "topic_comment"
    PHOTO_COMMENT = "photo_comment"
    VIDEO_COMMENT = "video_comment"
    MARKET = "market"
    MARKET_COMMENT = "market_comment"
    CLIP = "clip"


class BaseLinkButtonActionType(StrEnum, metaclass=BaseEnumMeta):  # type: ignore
    OPEN_URL = "open_url"
    JOIN_GROUP_AND_OPEN_URL = "join_group_and_open_url"
    MARKET_CLEAR_RECENT_QUERIES = "market_clear_recent_queries"
    CLOSE_WEB_APP = "close_web_app"
    ADD_PLAYLIST = "add_playlist"
    OPEN_SEARCH_TAB = "open_search_tab"
    OPEN_SEARCH_FILTERS = "open_search_filters"
    RESET_SEARCH_FILTERS = "reset_search_filters"
    IMPORT_CONTACTS = "import_contacts"
    ADD_FRIENDS = "add_friends"
    ONBOARDING = "onboarding"
    SHOW_FILTERS = "show_filters"


class GroupsUserXtrRole(UsersUserFull, GroupsMemberRole):  # type: ignore
    pass


class MessagesMessageAction(MessagesMessageAction):  # type: ignore
    type: "MessagesMessageActionStatus"  # type: ignore
    style: str | None = None


class GroupCallInProgress(CallsCall):
    receiver_id: int | None = None  # type: ignore
    time: int | None = None  # type: ignore
    join_link: str | None = None
    state: "CallsEndState | None" = None  # type: ignore


class AppsApp(AppsApp):  # type: ignore
    id: int | None = None  # type: ignore
    type: str | None = None  # type: ignore


class MessagesAudioMessage(MessagesAudioMessage):  # type: ignore
    transcript_state: str | None = None
    transcript: str | None = None


class BaseLinkAttachment(BaseLink):
    photo: "LinkPhoto | None" = None


class LinkPhoto(PhotosPhoto):
    has_tags: bool | None = None  # type: ignore
    date: int | None = None  # type: ignore


class PhotosPhotoAlbumFull(PhotosPhotoAlbumFull):  # type: ignore
    created: int | None = None
    updated: int | None = None


class PhotosPhoto(PhotosPhoto):  # type: ignore
    has_tags: bool | None = None  # type: ignore


class MessagesMessage(MessagesMessage):  # type: ignore
    attachments: list["MessagesMessageAttachment"] | None = None
    reply_message: "MessagesForeignMessage | None" = None
    fwd_messages: list["MessagesForeignMessage"] | None = None  # type: ignore


class MessagesForeignMessage(MessagesForeignMessage):  # type: ignore
    attachments: list["MessagesMessageAttachment"] | None = None
    reply_message: "MessagesForeignMessage | None" = None
    fwd_messages: list["MessagesForeignMessage"] | None = None


class MessagesPinnedMessage(MessagesPinnedMessage):  # type: ignore
    attachments: list["MessagesMessageAttachment"] | None = None
    reply_message: "MessagesForeignMessage | None" = None
    fwd_messages: list["MessagesForeignMessage"] | None = None


class VideoVideo(VideoVideo):  # type: ignore
    type: "VideoVideoType | None" = None


class VideoVideoFull(VideoVideo, VideoVideoFull):  # type: ignore
    type: "VideoVideoType | None" = None


class MessagesSendUserIdsResponseItem(BaseModel):  # type: ignore
    conversation_message_id: int | None = None
    error: "BaseMessageError | None" = None
    message_id: int | None
    peer_id: int


class MessagesMessageAttachment(MessagesMessageAttachment):  # type: ignore
    type: "MessagesMessageAttachmentType"  # type: ignore
    audio_message: "MessagesAudioMessage | None" = None
    story: "StoriesStory | None" = None
    group_call_in_progress: "GroupCallInProgress | None" = None
    link: "BaseLinkAttachment | None" = None
    wall: "WallWallpostFull | None" = None
    photo: "PhotosPhoto | None" = None
    mini_app: "AppsApp | None" = None
    sticker: "BaseSticker | None" = None
    video: "VideoVideoFull | None" = None
    poll: "PollsPoll | None" = None
    # NOTE: Add `narrative` field. Now, we've no docs and schema about this attachment.


class BaseLinkNoProduct(BaseLinkNoProduct):  # type: ignore
    photo: "PhotosPhoto | None" = None


class BaseCropPhoto(BaseCropPhoto):  # type: ignore
    photo: "PhotosPhoto" = Field()


class BugtrackerAttachment(BugtrackerAttachment):  # type: ignore
    photo: "PhotosPhoto | None" = None


class CallbackGroupChangePhoto(CallbackGroupChangePhoto):  # type: ignore
    photo: "PhotosPhoto" = Field()


class MarketMarketAlbum(MarketMarketAlbum):  # type: ignore
    photo: "PhotosPhoto | None" = None


class MarketOrderItem(MarketOrderItem):  # type: ignore
    photo: "PhotosPhoto | None" = None


class MessagesHistoryMessageAttachment(MessagesHistoryMessageAttachment):  # type: ignore
    photo: "PhotosPhoto | None" = None


class PhotosPhotoAlbum(PhotosPhotoAlbum):  # type: ignore
    thumb: "PhotosPhoto | None" = None


class StoriesStory(StoriesStory):  # type: ignore
    photo: "PhotosPhoto | None" = None


class WallCommentAttachment(WallCommentAttachment):  # type: ignore
    photo: "PhotosPhoto | None" = None


class WallWallpostAttachment(WallWallpostAttachment):  # type: ignore
    photo: "PhotosPhoto | None" = None


class NewsfeedItemPhotoPhotos(NewsfeedItemPhotoPhotos):  # type: ignore
    items: list["PhotosPhoto"] | None = None


class NewsfeedItemPhotoTagPhotoTags(NewsfeedItemPhotoTagPhotoTags):  # type: ignore
    items: list["PhotosPhoto"] | None = None


class UsersUserFull(UsersUserFull):  # type: ignore
    photo_max_size: "PhotosPhoto | None" = None


class MarketMarketItemFull(MarketMarketItemFull):  # type: ignore
    photos: "PhotosPhoto | None" = None


class MessagesKeyboardButtonPropertyAction(MessagesKeyboardButtonPropertyAction):  # type: ignore
    label: str = Field()
    type: str = Field()
    payload: str = Field()


class StoriesClickableSticker(StoriesClickableSticker):  # type: ignore[no-redef]
    poll: "PollsPoll | None" = None


class ClientInfoForBots(BaseModel):
    """
    Model: `ClientInfoForBots`
    """

    button_actions: list["MessagesTemplateActionTypeNames"] | None = Field(
        default=None,
    )
    """Property `ClientInfoForBots.button_actions`."""

    keyboard: bool | None = Field(
        default=None,
    )
    """client has support keyboard."""

    inline_keyboard: bool | None = Field(
        default=None,
    )
    """client has support inline keyboard."""

    carousel: bool | None = Field(
        default=None,
    )
    """client has support carousel."""

    lang_id: int | None = Field(
        default=None,
    )
    """client or user language id."""


UsersSubscriptionsItem: TypeAlias = GroupsGroupFull | UsersUserFull  # type: ignore


localns = locals().copy()
for item in localns.values():
    if not (isinstance(item, type) and item is not BaseModel and issubclass(item, BaseModel)):
        continue

    item.model_rebuild(force=True, _types_namespace=localns)

    for parent in item.__bases__:
        if parent.__name__ == item.__name__ and issubclass(parent, BaseModel):
            parent.__pydantic_fields__.update(item.__pydantic_fields__)
            parent.model_rebuild(force=True, _types_namespace=localns)
            item.__pydantic_fields__.update(
                {name: field for name, field in parent.__pydantic_fields__.items() if name not in item.__pydantic_fields__},
            )



from typing import TYPE_CHECKING

from vkbottle_types.codegen.methods.secure import SecureCategory as _SecureCategory  # type: ignore
from vkbottle_types.responses.secure import (
    SecureSetCounterArrayResponse,
    SecureSetCounterIntegerResponse,
    SecureSetCounterItem,
)

if TYPE_CHECKING:
    from vkbottle import ABCAPI  # type: ignore


class SecureCategory(_SecureCategory):  # type: ignore
    def __init__(self, api: "ABCAPI") -> None:
        super().__init__(api)

    async def set_counter(  # type: ignore
        self,
        counters: list[str] | None = None,
        user_id: int | None = None,
        counter: int | None = None,
        increment: bool | None = None,
        **kwargs,
    ) -> int | list["SecureSetCounterItem"]:
        """Sets a counter which is shown to the user in bold in the left menu.

        :param counters:
        :param user_id:
        :param counter: counter value.
        :param increment:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("secure.setCounter", params)
        model = SecureSetCounterArrayResponse if counters and counters.count(",") > 0 else SecureSetCounterIntegerResponse
        return model(**response).response


__all__ = ("SecureCategory",)



import typing

from vkbottle_types.methods.base_category import BaseCategory
from vkbottle_types.objects import (
    FriendsFriendExtendedStatus,
    FriendsFriendStatus,
    FriendsMutualFriend,
    FriendsOnlineUsers,
    FriendsOnlineUsersWithMobile,
    UsersFields,
)
from vkbottle_types.responses.base import (
    BaseOkResponse,
    BaseOkResponseModel,
)
from vkbottle_types.responses.friends import *  # noqa: F401,F403  # type: ignore


class FriendsCategory(BaseCategory):
    async def add(
        self,
        follow: bool | None = None,
        text: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsAddResponseModel:
        """Method `friends.add()`

        :param follow: '1' to pass an incoming request to followers list.
        :param text: Text of the message (up to 500 characters) for the friend request, if any.
        :param user_id: ID of the user whose friend request will be approved or to whom a friend request will be sent.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.add", params)
        model = FriendsAddResponse
        return model(**response).response

    async def add_list(
        self,
        name: str,
        user_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> FriendsAddListResponseModel:
        """Method `friends.addList()`

        :param name: Name of the friend list.
        :param user_ids: IDs of users to be added to the friend list.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.addList", params)
        model = FriendsAddListResponse
        return model(**response).response

    @typing.overload
    async def are_friends(
        self,
        user_ids: list[int],
        extended: typing.Literal[True],
        need_sign: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[FriendsFriendExtendedStatus]: ...

    @typing.overload
    async def are_friends(
        self,
        user_ids: list[int],
        extended: typing.Literal[False] | None = None,
        need_sign: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[FriendsFriendStatus]: ...

    async def are_friends(
        self,
        user_ids: list[int],
        extended: bool | None = None,
        need_sign: bool | None = None,
        **kwargs: typing.Any,
    ) -> list[FriendsFriendStatus] | list[FriendsFriendExtendedStatus]:
        """Method `friends.areFriends()`

        :param user_ids: IDs of the users whose friendship status to check.
        :param extended: Return friend request read_state field
        :param need_sign: '1' - to return 'sign' field. 'sign' is md5("{id}_{user_id}_{friends_status}_{application_secret}"), where id is current user ID. This field allows to check that data has not been modified by the client. By default: '0'.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.areFriends", params)
        model = self.get_model(
            ((("extended",), FriendsAreFriendsExtendedResponse),),
            default=FriendsAreFriendsResponse,
            params=params,
        )
        return model(**response).response

    async def delete(
        self,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsDeleteResponseModel:
        """Method `friends.delete()`

        :param user_id: ID of the user whose friend request is to be declined or who is to be deleted from the current user's friend list.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.delete", params)
        model = FriendsDeleteResponse
        return model(**response).response

    async def delete_all_requests(
        self,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `friends.deleteAllRequests()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("friends.deleteAllRequests", params)
        model = BaseOkResponse
        return model(**response).response

    async def delete_list(
        self,
        list_id: int,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `friends.deleteList()`

        :param list_id: ID of the friend list to delete.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.deleteList", params)
        model = BaseOkResponse
        return model(**response).response

    async def edit(
        self,
        user_id: int,
        list_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `friends.edit()`

        :param user_id: ID of the user whose friend list is to be edited.
        :param list_ids: IDs of the friend lists to which to add the user.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.edit", params)
        model = BaseOkResponse
        return model(**response).response

    async def edit_list(
        self,
        list_id: int,
        add_user_ids: list[int] | None = None,
        delete_user_ids: list[int] | None = None,
        name: str | None = None,
        user_ids: list[int] | None = None,
        **kwargs: typing.Any,
    ) -> BaseOkResponseModel:
        """Method `friends.editList()`

        :param list_id: Friend list ID.
        :param add_user_ids: (Applies if 'user_ids' parameter is not set.), User IDs to add to the friend list.
        :param delete_user_ids: (Applies if 'user_ids' parameter is not set.), User IDs to delete from the friend list.
        :param name: Name of the friend list.
        :param user_ids: IDs of users in the friend list.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.editList", params)
        model = BaseOkResponse
        return model(**response).response

    @typing.overload
    async def get(
        self,
        fields: list[UsersFields],
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        ref: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetFieldsResponseModel: ...

    @typing.overload
    async def get(
        self,
        fields: list[UsersFields] | None = None,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        ref: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetResponseModel: ...

    async def get(
        self,
        fields: list[UsersFields] | None = None,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        ref: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetResponseModel | FriendsGetFieldsResponseModel:
        """Method `friends.get()`

        :param fields: Profile fields to return. Sample values: 'uid', 'first_name', 'last_name', 'nickname', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'domain', 'has_mobile', 'rate', 'contacts', 'education'.
        :param count: Number of friends to return.
        :param list_id: ID of the friend list returned by the [vk.com/dev/friends.getLists|friends.getLists] method to be used as the source. This parameter is taken into account only when the uid parameter is set to the current user ID. This parameter is available only for [vk.com/dev/standalone|desktop applications].
        :param offset: Offset needed to return a specific subset of friends.
        :param order: Sort order: , 'name' - by name (enabled only if the 'fields' parameter is used), 'hints' - by rating, similar to how friends are sorted in My friends section, , This parameter is available only for [vk.com/dev/standalone|desktop applications].
        :param ref:
        :param user_id: User ID. By default, the current user ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.get", params)
        model = self.get_model(
            ((("fields",), FriendsGetFieldsResponse),),
            default=FriendsGetResponse,
            params=params,
        )
        return model(**response).response

    async def get_app_users(
        self,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `friends.getAppUsers()`"""

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getAppUsers", params)
        model = FriendsGetAppUsersResponse
        return model(**response).response

    async def get_lists(
        self,
        return_system: bool | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetListsResponseModel:
        """Method `friends.getLists()`

        :param return_system: '1' - to return system friend lists. By default: '0'.
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getLists", params)
        model = FriendsGetListsResponse
        return model(**response).response

    @typing.overload
    async def get_mutual(
        self,
        target_uids: list[int],
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
        **kwargs: typing.Any,
    ) -> list[FriendsMutualFriend]: ...

    @typing.overload
    async def get_mutual(
        self,
        target_uids: list[int] | None = None,
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
        **kwargs: typing.Any,
    ) -> "FriendsMutualFriend": ...

    @typing.overload
    async def get_mutual(
        self,
        target_uids: list[int],
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
        **kwargs: typing.Any,
    ) -> list[int]: ...

    async def get_mutual(
        self,
        target_uids: list[int] | None = None,
        count: int | None = None,
        need_common_count: bool | None = None,
        offset: int | None = None,
        order: str | None = None,
        source_uid: int | None = None,
        target_uid: int | None = None,
        **kwargs: typing.Any,
    ) -> "list[int] | FriendsMutualFriend | list[FriendsMutualFriend]":
        """Method `friends.getMutual()`

        :param target_uids: IDs of the users whose friends will be checked against the friends of the user specified in 'source_uid'.
        :param count: Number of mutual friends to return.
        :param need_common_count: Return mutual friends total count
        :param offset: Offset needed to return a specific subset of mutual friends.
        :param order: Sort order: 'random' - random order
        :param source_uid: ID of the user whose friends will be checked against the friends of the user specified in 'target_uid'.
        :param target_uid: ID of the user whose friends will be checked against the friends of the user specified in 'source_uid'.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getMutual", params)
        model = self.get_model(
            (
                (("target_uids",), FriendsGetMutualTargetUidsResponse),
                (("total_count",), FriendsGetMutualTotalCountResponse),
            ),
            default=FriendsGetMutualResponse,
            params=params,
        )
        return model(**response).response

    @typing.overload
    async def get_online(
        self,
        online_mobile: typing.Literal[True],
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetOnlineOnlineMobileResponseModel: ...

    @typing.overload
    async def get_online(
        self,
        online_mobile: bool | None = None,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "FriendsOnlineUsers": ...

    @typing.overload
    async def get_online(
        self,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        online_mobile: bool | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "FriendsOnlineUsersWithMobile": ...

    @typing.overload
    async def get_online(
        self,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        online_mobile: bool | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> list[int]: ...

    async def get_online(
        self,
        online_mobile: bool | None = None,
        count: int | None = None,
        list_id: int | None = None,
        offset: int | None = None,
        order: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> "FriendsOnlineUsersWithMobile | FriendsOnlineUsers | list[int] | FriendsGetOnlineOnlineMobileResponseModel":
        """Method `friends.getOnline()`

        :param online_mobile: '1' - to return an additional 'online_mobile' field, '0' - (default),
        :param count: Number of friends to return.
        :param list_id: Friend list ID. If this parameter is not set, information about all online friends is returned.
        :param offset: Offset needed to return a specific subset of friends.
        :param order: Sort order: 'random' - random order
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getOnline", params)
        model = self.get_model(
            (
                (("online_mobile",), FriendsGetOnlineOnlineMobileResponse),
                (("extended",), FriendsGetOnlineExtendedResponse),
                (("online_mobile_extended",), FriendsGetOnlineOnlineMobileExtendedResponse),
            ),
            default=FriendsGetOnlineResponse,
            params=params,
        )
        return model(**response).response

    async def get_recent(
        self,
        count: int | None = None,
        **kwargs: typing.Any,
    ) -> list[int]:
        """Method `friends.getRecent()`

        :param count: Number of recently added friends to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getRecent", params)
        model = FriendsGetRecentResponse
        return model(**response).response

    @typing.overload
    async def get_requests(
        self,
        need_mutual: typing.Literal[True],
        extended: typing.Literal[True],
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetRequestsNeedMutualResponseModel: ...

    @typing.overload
    async def get_requests(
        self,
        need_mutual: bool | None = None,
        extended: bool | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetRequestsExtendedResponseModel: ...

    @typing.overload
    async def get_requests(
        self,
        count: int | None = None,
        extended: bool | None = None,
        fields: list[UsersFields] | None = None,
        need_mutual: bool | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetRequestsResponseModel: ...

    async def get_requests(
        self,
        extended: bool | None = None,
        need_mutual: bool | None = None,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        need_viewed: bool | None = None,
        offset: int | None = None,
        out: bool | None = None,
        ref: str | None = None,
        sort: int | None = None,
        suggested: bool | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetRequestsResponseModel | FriendsGetRequestsExtendedResponseModel | FriendsGetRequestsNeedMutualResponseModel:
        """Method `friends.getRequests()`

        :param extended: '1' - to return response messages from users who have sent a friend request or, if 'suggested' is set to '1', to return a list of suggested friends
        :param need_mutual: '1' - to return a list of mutual friends (up to 20), if any
        :param count: Number of friend requests to return (default 100, maximum 1000).
        :param fields:
        :param need_viewed:
        :param offset: Offset needed to return a specific subset of friend requests.
        :param out: '1' - to return outgoing requests, '0' - to return incoming requests (default)
        :param ref:
        :param sort: Sort order: '1' - by number of mutual friends, '0' - by date
        :param suggested: '1' - to return a list of suggested friends, '0' - to return friend requests (default)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getRequests", params)
        model = self.get_model(
            (
                (("need_mutual",), FriendsGetRequestsNeedMutualResponse),
                (("extended",), FriendsGetRequestsNeedMutualResponse),
            ),
            default=FriendsGetRequestsResponse,
            params=params,
        )
        return model(**response).response

    async def get_suggestions(
        self,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        filter: list[typing.Literal["contacts", "mutual", "mutual_contacts"]] | None = None,
        name_case: str | None = None,
        offset: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsGetSuggestionsResponseModel:
        """Method `friends.getSuggestions()`

        :param count: Number of suggestions to return.
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online', 'counters'.
        :param filter: Types of potential friends to return: 'mutual' - users with many mutual friends , 'contacts' - users found with the [vk.com/dev/account.importContacts|account.importContacts] method , 'mutual_contacts' - users who imported the same contacts as the current user with the [vk.com/dev/account.importContacts|account.importContacts] method
        :param name_case: Case for declension of user name and surname: , 'nom' - nominative (default) , 'gen' - genitive , 'dat' - dative , 'acc' - accusative , 'ins' - instrumental , 'abl' - prepositional
        :param offset: Offset needed to return a specific subset of suggestions.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.getSuggestions", params)
        model = FriendsGetSuggestionsResponse
        return model(**response).response

    async def search(
        self,
        count: int | None = None,
        fields: list[UsersFields] | None = None,
        name_case: str | None = None,
        offset: int | None = None,
        q: str | None = None,
        user_id: int | None = None,
        **kwargs: typing.Any,
    ) -> FriendsSearchResponseModel:
        """Method `friends.search()`

        :param count: Number of friends to return.
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online',
        :param name_case: Case for declension of user name and surname: 'nom' - nominative (default), 'gen' - genitive , 'dat' - dative, 'acc' - accusative , 'ins' - instrumental , 'abl' - prepositional
        :param offset: Offset needed to return a specific subset of friends.
        :param q: Search query string (e.g., 'Vasya Babich').
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("friends.search", params)
        model = FriendsSearchResponse
        return model(**response).response


__all__ = ("FriendsCategory",)



from vkbottle_types.base_model import BaseModel, Field
from vkbottle_types.objects import (
    AppsApp,
    AppsCatalogList,
    AppsCustomSnippet,
    AppsLeaderboard,
    AppsScope,
    AppsTestingGroup,
    GroupsGroupFull,
    UsersUser,
    UsersUserFull,
)
from vkbottle_types.responses.base_response import BaseResponse


class AppsAddSnippetResponseModel(BaseModel):
    snippet_id: int = Field()


class AppsAddSnippetResponse(BaseResponse):
    response: "AppsAddSnippetResponseModel" = Field()


class AppsCreatedGroupResponseModel(BaseModel):
    group_id: int = Field()


class AppsCreatedGroupResponse(BaseResponse):
    response: "AppsCreatedGroupResponseModel" = Field()


class AppsGetCatalogResponse(BaseResponse):
    response: "AppsCatalogList" = Field()


class AppsGetFriendsListExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list["UsersUserFull"] = Field()


class AppsGetFriendsListExtendedResponse(BaseResponse):
    response: "AppsGetFriendsListExtendedResponseModel" = Field()


class AppsGetFriendsListResponseModel(BaseModel):
    count: int = Field()
    items: list[int] = Field()


class AppsGetFriendsListResponse(BaseResponse):
    response: "AppsGetFriendsListResponseModel" = Field()


class AppsGetLeaderboardExtendedResponseModel(BaseModel):
    count: int = Field()
    items: list["AppsLeaderboard"] = Field()
    profiles: list["UsersUser"] | None = Field(
        default=None,
    )


class AppsGetLeaderboardExtendedResponse(BaseResponse):
    response: "AppsGetLeaderboardExtendedResponseModel" = Field()


class AppsGetLeaderboardResponseModel(BaseModel):
    count: int = Field()
    items: list["AppsLeaderboard"] = Field()


class AppsGetLeaderboardResponse(BaseResponse):
    response: "AppsGetLeaderboardResponseModel" = Field()


class AppsGetMiniAppPoliciesResponseModel(BaseModel):
    privacy_policy: str | None = Field(
        default=None,
    )
    terms: str | None = Field(
        default=None,
    )


class AppsGetMiniAppPoliciesResponse(BaseResponse):
    response: "AppsGetMiniAppPoliciesResponseModel" = Field()


class AppsGetScopesResponseModel(BaseModel):
    count: int = Field()
    items: list["AppsScope"] = Field()


class AppsGetScopesResponse(BaseResponse):
    response: "AppsGetScopesResponseModel" = Field()


class AppsGetScoreResponse(BaseResponse):
    response: int = Field()


class AppsGetSnippetsResponseModel(BaseModel):
    items: list["AppsCustomSnippet"] | None = Field(
        default=None,
    )


class AppsGetSnippetsResponse(BaseResponse):
    response: "AppsGetSnippetsResponseModel" = Field()


class AppsGetTestingGroupsResponse(BaseResponse):
    response: list["AppsTestingGroup"] = Field()


class AppsGetResponseModel(BaseModel):
    count: int = Field()
    items: list["AppsApp"] = Field()
    profiles: list["UsersUserFull"] | None = Field(
        default=None,
    )
    groups: list["GroupsGroupFull"] | None = Field(
        default=None,
    )


class AppsGetResponse(BaseResponse):
    response: "AppsGetResponseModel" = Field()


class AppsImageUploadResponseModel(BaseModel):
    hash: str | None = Field(
        default=None,
    )
    image: str | None = Field(
        default=None,
    )


class AppsImageUploadResponse(BaseResponse):
    response: "AppsImageUploadResponseModel" = Field()


class AppsIsNotificationsAllowedResponseModel(BaseModel):
    is_allowed: bool = Field()


class AppsIsNotificationsAllowedResponse(BaseResponse):
    response: "AppsIsNotificationsAllowedResponseModel" = Field()


class AppsSendRequestResponse(BaseResponse):
    response: int = Field()



import builtins
import json  # type: ignore
import os
import pathlib
import typing

import jinja2
import requests  # type: ignore
from tqdm import tqdm  # type: ignore

from . import parse_types
from .data import Category, Definition, Objects, factory
from .parse_types import get_complex_type, transform_ref  # noqa: F401
from .utility import camelcase, instring, makesafe, makesafe_method_name, snake_case

URL = "https://raw.githubusercontent.com/VKCOM/vk-api-schema/master/{}/{}.json"
CATEGORIES = (
    "base",
    "users",
    "messages",
    "account",
    "address",
    "addresses",
    "ads",
    "appWidgets",
    "apps",
    "audio",
    "auth",
    "board",
    "bugtracker",
    "callback",
    "calls",
    "client",
    "comment",
    "database",
    "docs",
    "donut",
    "downloadedGames",
    "events",
    "fave",
    "gifts",
    "groups",
    "leadForms",
    "likes",
    "link",
    "market",
    "notes",
    "notifications",
    "oauth",
    "orders",
    "owner",
    "pages",
    "photos",
    "podcast",
    "podcasts",
    "polls",
    "prettyCards",
    "search",
    "secure",
    "stats",
    "status",
    "stickers",
    "storage",
    "store",
    "stories",
    "streaming",
    "support",
    "tasks",
    "translations",
    "friends",
    "utils",
    "video",
    "wall",
    "newsfeed",
    "widgets",
)

env = jinja2.Environment(loader=jinja2.FileSystemLoader(pathlib.Path(__file__).parent / "templates"))
env.globals.update(
    {
        "snake_case": snake_case,
        "camelcase": camelcase,
        "is_int": lambda x: isinstance(x, int),
        "makesafe": makesafe,
        "makesafe_method_name": makesafe_method_name,
        "instring": instring,
        "unique": lambda iterable: set(iterable),
    }
)


def process_downloaded_category(dct: dict) -> Category:
    for key in ("methods", "objects", "responses"):
        if not dct.get(key):
            dct.pop(key, None)

    if methods := dct.get("methods", {}).get("methods"):
        for method in methods:
            method["name"] = method["name"].split(".")[1]
        dct["methods"] = methods

    category = factory.load(dct, Category)
    return category


def process_schema(categories_dcts: list[dict]) -> list[Category]:
    print("Processing schema...")
    categories = []
    for dct in tqdm(categories_dcts):
        if not dct:
            continue
        processed = process_downloaded_category(dct)
        categories.append(processed)
    return categories


def download_schema() -> list[Category]:
    print("Downloading schema...")

    categories_dcts = []
    for category_name in tqdm(CATEGORIES):
        dct: dict[str, typing.Any] = {"name": category_name}
        for file in ("methods", "objects", "responses"):
            result = requests.get(URL.format(category_name, file))
            if result.status_code == 200:
                dct[file] = result.json()
            else:
                dct[file] = None
        categories_dcts.append(dct)

    with open("original-schema.json", "w") as fp:
        json.dump(categories_dcts, fp)

    return process_schema(categories_dcts)


def save_schema(schema: list[Category], filename: str = "schema.json") -> None:
    with open(filename, "w") as fp:
        json.dump([factory.dump(c) for c in schema], fp)  # type: ignore


def load_schema(filename: str = "schema.json") -> list[Category]:
    with open(filename, "r") as fp:
        data = json.load(fp)
        return [factory.load(c, Category) for c in data]


def load_and_save() -> list[Category]:
    path = pathlib.Path(input("Enter path to schema: "))
    if not path.exists():
        print(f"Path ({path}) does not exist!")
        exit(0)
    with open(path) as file:
        schema_data = json.load(file)
    schema = process_schema(schema_data)
    save_schema(schema)
    return schema


def get_schema_cli() -> list[Category]:
    commands = {
        1: "Use it [default]",
        2: "Download it once again",
        3: "Process from path and rewrite",
    }
    if pathlib.Path("schema.json").exists():
        option = input("Existing 'schema.json' found.\n    " + "\n    ".join(["({}) {}".format(i, s) for i, s in commands.items()]) + "\n: ")

        if not option.strip() or int(option) == 1:
            return load_schema()

        match int(option):
            case 2:
                schema = download_schema()
                save_schema(schema)
            case 3:
                schema = load_and_save()
        return schema

    commands = {
        1: "Download official schema [default]",
        2: "Process from path and save",
    }

    option = input(
        "Should new schema be downloaded from source?\n    " + "\n    ".join(["({}) {}".format(i, s) for i, s in commands.items()]) + "\n: "
    )

    if not option.strip():
        option = 1  # type: ignore

    match int(option):
        case 1:
            schema = download_schema()
            save_schema(schema)
        case 2:
            schema = load_and_save()

    return schema


def apply_patches(schema: list[Category]) -> None:
    print("Applying patches...")
    # TODO


def create_folders(folders: list[tuple[str, ...]]) -> None:
    for folder in folders:
        path = pathlib.Path(*folder)
        path.mkdir(exist_ok=True)
        (path / "__init__.py").touch(exist_ok=True)


def generate_methods(category: Category, path: str):
    if not category.methods:
        print("        - Methods not found!", end="\n\n")
        return

    imports = set()
    for method in category.methods:
        for response in typing.cast(dict, method.responses).values():
            if "response_hint" not in response:
                continue
            for t in response["response_hint"]["orig_types"]:
                if t["is_object"]:
                    imports.add(t["type"])
        for param in method.parameters:
            if "$ref" in param.items and "objects.json" in param.items["$ref"]:
                imports.add(transform_ref(param.items["$ref"]))

    template = env.get_template("methods.jinja2")
    generated = template.render(category=category)
    generated = generated.replace(
        "### IMPORTS",
        ("from vkbottle_types.objects import " + ", ".join(imports)) if imports else "",
    )
    pathlib.Path(path, "methods", snake_case(category.name) + ".py").write_text(generated, encoding="UTF-8")
    print("        + Methods successfully generated!", end="\n\n")


def generate_responses(category: Category, path: str) -> None:
    if not category.responses:
        print("        - Responses not found!")
        return

    definitions = get_definitions(category.responses)
    parse_types.IMPORTS_CACHE = set()
    template = env.get_template("responses.jinja2")
    generated = template.render(definitions=definitions)
    generated = generated.replace(
        "### IMPORTS",
        ("from vkbottle_types.objects import " + ", ".join(parse_types.IMPORTS_CACHE)) if parse_types.IMPORTS_CACHE else "",
    )
    pathlib.Path(path, "responses", snake_case(category.name) + ".py").write_text(generated, encoding="UTF-8")
    print("        + Responses successfully generated!")


def process_responses(responses: dict[str, dict[str, str]], responses_definitions: dict[str, Definition]) -> dict[str, dict[str, str]]:
    for response_name, response in responses.items():
        if "$ref" not in response:
            continue
        if (definition := responses_definitions.get(response["$ref"].split("/")[-1])) is not None:
            for prop in definition.properties:
                if prop.name == "response":
                    if (response_definition := definition.get_response_definition()) is not None:
                        if response_definition.properties or response_definition.enum:
                            hint = get_complex_type(response, response=True)
                        else:
                            hint = "dict[str, typing.Any]"
                    else:
                        hint = prop.get_type(response=True)

                    orig_type: str | list[str]
                    if "Literal[" in hint:
                        orig_type = hint
                    else:
                        orig_type = hint.replace("'", "").replace('"', "").replace("list", "").replace("[", "").replace("]", "").strip()
                        if "|" in orig_type:
                            orig_type = list(map(str.strip, orig_type.split(" | ")))
                        if "dict" in orig_type:
                            orig_type = "dict"
                    responses[response_name]["response_hint"] = {  # type: ignore
                        "hint": hint,
                        "orig_types": [
                            {
                                "type": t,
                                "is_object": is_object(t, hint),
                            }
                            for t in orig_type
                        ]
                        if isinstance(orig_type, list)
                        else [
                            {
                                "type": orig_type,
                                "is_object": is_object(orig_type, hint),
                            }
                        ],
                    }

    return responses.copy()


def update_category_methods_responses(category: Category, responses_definitions: dict[str, Definition]) -> None:
    for method in category.methods:
        responses = typing.cast(dict[str, dict[str, str]], method.responses)
        responses.update(process_responses(responses, responses_definitions))


def reorder_definitions(
    definitions: list[tuple[str, Definition, Category]],
) -> list[tuple[str, Definition, Category]]:
    l_bases_f = lambda d: len(d[1].allOf)  # noqa: E731
    s = sorted(definitions, key=l_bases_f)
    s = sorted(
        definitions,
        key=lambda d: 0
        if not l_bases_f(d)
        else (1 + CATEGORIES.index(d[2].name) + any(b.lower().startswith(d[2].name.lower()) for b in d[1].get_bases())),
    )
    return s


def generate_objects(definitions: list[tuple[str, Definition, Category]], path: str) -> None:
    definitions_dct = dict([(k, v) for (k, v, _) in reorder_definitions(definitions)])

    for definition in definitions_dct.values():
        if definition.type == "object" and not definition.properties and definition.ref:
            definition.properties.extend(definitions_dct[definition.ref.split("/")[-1]].properties)

    template = env.get_template("objects.jinja2")
    generated = template.render(definitions=definitions_dct)
    pathlib.Path(path, "objects.py").write_text(generated, encoding="UTF-8")


def is_object(t: str, hint: str) -> bool:
    return t not in builtins.__dict__ and "Response" not in t and "Literal[" not in hint


def generate_category(category: Category, path: str, responses_definitions: dict[str, Definition]):
    generate_responses(category, path)
    update_category_methods_responses(category, responses_definitions)
    generate_methods(category, path)


def parse_properties(properties: dict[str, dict]) -> list[dict]:
    return [{"name": name, **prop, "data": prop} for name, prop in properties.items()]


def get_definition(
    name: str,
    definitions: typing.Iterable[typing.Tuple[str, Definition]],
) -> Definition | None:
    for definition_name, definition in definitions:
        if definition_name == name:
            return definition
    return None


def get_definitions(objects: Objects) -> list[tuple[str, Definition]] | None:
    if not objects.definitions:
        return []

    definitions: dict[str, Definition] = {}
    for definition_name, definition in typing.cast(dict[str, dict], objects.definitions).items():
        if "properties" in definition:
            properties: dictt[str, dict] = {}

            if isinstance(definition["properties"], dict):
                properties = definition["properties"]
            else:
                for prop in definition["properties"]:
                    properties[prop.pop("name")] = prop

            definition["properties"] = parse_properties(properties)

        definitions.update({definition_name: factory.load(definition, Definition)})

    return list(definitions.items())


def generate_schema(schema: list[Category], folder: str) -> None:
    create_folders([(folder,), (folder, "methods"), (folder, "responses")])

    print("Generating schema...")

    responses_definitions: dict[str, Definition] = {}
    for c in schema:
        if c.responses and c.responses.definitions:
            responses_definitions.update(get_definitions(c.responses))

    for category in schema:
        print(f"    * Generating category: {category.name!r}...")
        generate_category(category, folder, responses_definitions)

    definitions: dict[str, tuple[Definition, Category]] = {}
    for category in schema:
        if not category.objects:
            continue

        definitions.update(
            {k: (v, category) for k, v in get_definitions(category.objects)},
        )

    sub_definitions: dict[str, list[str]] = {}
    for def_name, (def_, _) in definitions.items():
        for base in def_.allOf:
            if base.ref:
                name = base.ref.split("/")[-1]
                if name not in sub_definitions:
                    sub_definitions[name] = []
                if def_name not in sub_definitions[name]:
                    sub_definitions[name].append(def_name)
                base.definition = get_definition(name, list((n, x[0]) for n, x in definitions.items()))

    for definition_name, (def_, _) in definitions.items():
        if (sub_defs := sub_definitions.get(definition_name)) is None:
            continue
        for def_name in sub_defs:
            if (d := definitions.get(def_name)) is None:
                continue
            def_.sub_definitions[camelcase(def_name)] = d[0]

    generate_objects([(k, *v) for k, v in definitions.items()], folder)

    print("Run ruff isort...")
    os.system(f"ruff check {folder} --select I --select F401 --fix")

    print("Run ruff sortall...")
    os.system(f"ruff check {folder} --select RUF022 --fix")

    print("Run fuff formatter...")
    os.system(f"ruff format {folder}")


__all__ = (
    "get_schema_cli",
    "generate_schema",
    "apply_patches",
    "save_schema",
    "load_schema",
    "process_schema",
    "download_schema",
    "Category",
    "Definition",
    "Objects",
    "factory",
)



import typing

from vkbottle_types.codegen.responses.utils import *  # noqa: F403,F401  # type: ignore


class UtilsResolveScreenNameResponse(BaseResponse):  # type: ignore
    response: "UtilsDomainResolved | list[typing.Any]" = Field(  # type: ignore
        default_factory=lambda: [],
    )



from typing import Any

import pydantic

from vkbottle_types.base_model import BaseModel, Field

PYDICT_APAPTER_TO_RAW_JSON = pydantic.TypeAdapter(dict[str, Any])


class BaseResponse(BaseModel):
    response: Any
    raw_json: str | None = Field(default=None)

    @property
    def raw(self) -> str:
        if not self.raw_json:
            raise AttributeError("You cannot get raw_json from here. Get a full raw_json from unnested response.")
        return self.raw_json


class DictResponse(BaseResponse):
    response: dict[str, Any]

    def __init__(self, **data: Any) -> None:
        super().__init__(response=data)

    @property
    def raw(self) -> str:  # type: ignore
        return PYDICT_APAPTER_TO_RAW_JSON.dump_json(self.response).decode()


__all__ = ("BaseResponse", "DictResponse")



import dataclasses
import re

from .utility import camelcase

PATTERN_PROPERTIES_RESPONSE = re.compile(r"^([a-zA-Z0-9_]+)?[rR]esponse$")

IMPORTS_CACHE = set()
UNIX_TIMESTAMP_DESCRIPTION_TEXT = "Unix timestamp"
PRIMITIVE_TYPES = {
    "integer": "int",
    "string": "str",
    "boolean": "bool",
    "number": "float",
    "Integer": "int",
    "String": "str",
    "Boolean": "bool",
    "Number": "float",
}


@dataclasses.dataclass
class Ready:
    value: str


def get_responses(responses: dict[str, dict[str, str]]) -> list[dict[str, str]]:
    if not responses:
        return []
    return [
        value
        for key, value in responses.items()
        if PATTERN_PROPERTIES_RESPONSE.match(key) is not None or (key != "response" and key.startswith("response"))
    ]


def transform_ref(ref_link: str) -> str:
    name = camelcase(ref_link.split("/")[-1])
    return name


def get_complex_type(type_dct: dict[str, object], response: bool = False, hint: bool = False) -> str:
    if (type_ := type_dct.get("type")) and type_ != "object":
        return get_type(type_, type_dct.get("items", type_dct), hint=hint)
    elif ref := type_dct.get("$ref"):
        transformed = transform_ref(ref)
        if response:
            if "response_hint" in type_dct:
                return type_dct["response_hint"]["hint"]
            return transformed + "Model"
        else:
            IMPORTS_CACHE.add(transformed)
        if hint:
            transformed = repr(transformed)
        return transformed

    raise RuntimeError(f"Cannot process complex type {type_dct}")


def get_type(
    type_name: str | list[str] | dict[str, object] | Ready,
    items: dict[str, object] | None = None,
    hint: bool = False,
) -> str:
    if isinstance(type_name, Ready):
        return repr(type_name.value) if hint else type_name.value

    items = items or {}

    match type_name:
        case "string":
            if literals := items.get("enum"):
                return "typing.Literal[{}]".format(", ".join(repr(literal) for literal in literals))
            return "str"
        case "array":
            return "list[{}]".format(get_complex_type(items, hint=hint))
        case "object":
            if "$ref" in items:
                return get_complex_type(items, hint=True)
            return "dict[str, typing.Any]"
        case "integer" if UNIX_TIMESTAMP_DESCRIPTION_TEXT in items.get("description", ""):
            return "datetime.datetime"
        case str(type_name):
            if type_name not in PRIMITIVE_TYPES:
                raise RuntimeError(f"{type_name} not in primitive types")
            return PRIMITIVE_TYPES[type_name]
        case list(lst):
            types = [get_type(name, {}, hint=False) for name in lst]
            union = " | ".join(t.strip("'").strip('"') for t in types)
            return "{}".format(f'"{union}"' if any("Literal" not in x and ("'" in x or '"' in x) for x in types) else union)
        case _:
            pass

    raise RuntimeError(f"Cannot process {type_name} (items={items})")


__all__ = ("get_type", "get_complex_type", "Ready", "transform_ref", "get_responses")
